//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/
// 
#include "stdafx.h"
#include "Qmol.h"
#include "QmolDoc.h"
#include "QmolView.h"
#include "AnimateDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAnimateDialog dialog


CAnimateDialog::CAnimateDialog(CView *pView)
{	
	// Save a pointer to the parent class
	m_pView = pView;

	//{{AFX_DATA_INIT(CAnimateDialog)
	m_frame_num = _T("");
	//}}AFX_DATA_INIT
}

BOOL CAnimateDialog::Create()
{
	return CDialog::Create(CAnimateDialog::IDD);
}

void CAnimateDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAnimateDialog)
	DDX_Control(pDX, IDC_ANIMATE_STOP, m_stop_button);
	DDX_Control(pDX, IDC_ANIMATE_REVERSE, m_reverse_button);
	DDX_Control(pDX, IDC_ANIMATE_FORWARD, m_forward_button);
	DDX_Control(pDX, IDC_ANIMATE_SLIDER, m_slider);
	DDX_Text(pDX, IDC_FRAME_NUM, m_frame_num);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CAnimateDialog, CDialog)
	//{{AFX_MSG_MAP(CAnimateDialog)
	ON_BN_CLICKED(IDC_ANIMATE_FORWARD, OnAnimateForward)
	ON_BN_CLICKED(IDC_ANIMATE_REVERSE, OnAnimateReverse)
	ON_BN_CLICKED(IDC_ANIMATE_STOP, OnAnimateStop)
	ON_WM_HSCROLL()
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAnimateDialog message handlers

void CAnimateDialog::OnAnimateForward() 
{
	if(m_pView != NULL){
		// Modeless case -- don't call the base class OnCancel
		m_pView->PostMessage(WM_ANIMATE_FORWARD);
	}
}

void CAnimateDialog::OnAnimateReverse() 
{
	if(m_pView != NULL){
		// Modeless case -- don't call the base class OnCancel
		m_pView->PostMessage(WM_ANIMATE_REVERSE);
	}
}

void CAnimateDialog::OnAnimateStop() 
{
	if(m_pView != NULL){
		// Modeless case -- don't call the base class OnCancel
		m_pView->PostMessage(WM_ANIMATE_STOP);
	}
}

BOOL CAnimateDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	m_reverse_button.SetIcon(AfxGetApp()->LoadIcon(IDI_ICON_REVERSE));
	m_forward_button.SetIcon(AfxGetApp()->LoadIcon(IDI_ICON_FORWARD));
	m_stop_button.SetIcon(AfxGetApp()->LoadIcon(IDI_ICON_STOP));

	CQmolDoc* pDoc = (CQmolDoc*)(m_pView->GetDocument());
	ASSERT_VALID(pDoc);

	if(pDoc->MaxTrajStep()){

		// Set the range and position of the slider
		m_slider.SetRange(0, pDoc->MaxTrajStep()); // [-Pi, Pi]
		m_slider.SetPos(pDoc->CurrentTrajStep());

		m_frame_num.Format("Frame: %d", 
			pDoc->CurrentTrajStep());
	}
	else{
		m_slider.SetPos(0);

		m_frame_num.Format("Frame: --");
	}

	UpdateData(FALSE);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CAnimateDialog::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	if(m_pView != NULL){
		m_pView->PostMessage(WM_ANIMATE_ADJUST);
	}
	
	CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
}

void CAnimateDialog::SetCurrentStep(int step)
{
	m_slider.SetPos(step);
	m_frame_num.Format("Frame: %d",step);
	UpdateData(FALSE);
}

void CAnimateDialog::OnClose() 
{
	if(m_pView != NULL){
		// Modeless case -- don't call the base class OnCancel
		m_pView->PostMessage(WM_ANIMATE_CLOSE, IDCANCEL);
	}
}
