//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/
// 

#ifndef __CLIP_PLANE
#define __CLIP_PLANE

// Include the OpenGL headers
#include <GL\gl.h>
#include <GL\glu.h>
#include "Excepts.h"
#include "trackball.h"
#include "QmolDoc.h"

// A class for managing clip planes in OpenGL

class ClipPlane: public VirtualTrackBall {
private:
	// Should we clip?
	BOOL enable_clipping;

	// Should the clip plane be displayed as a real plane?
	BOOL show_plane;
	
	// Which clip plane are we using?
	int clip_id;

	// The color of the clip plane
	GLfloat clip_plane_color[4];

	GLfloat center[3];
	GLfloat normal[3];		// Must have unit length
	GLfloat A[3], B[3];		// An orthogonal coordiante system:
							// [A, B, normal]

	// The coefficents of the clip plane:
	// eqn[0]*x + eqn[1]*y + eqn[2]*z + eqn[3] = 0.
	// Note that the coordinates of the clipplane are
	// related to the center and normal vectors by:
	// eqn[0] = normal[0]
	// eqn[1] = normal[1]
	// eqn[2] = normal[2]
	// eqn[3] = normal.center,
	// where the normal vector has unit length.
	GLdouble eqn[4];

	// Track rotation and scaling
	int begin_rotation_x;
	int begin_rotation_y;
	int begin_scale_x;

	// A correction for clipping stereo views
	GLfloat stereo_rot;

	// Compute the coordiante system: A, B, normal
	void UpdateCoordSystem();

	// Do we need to display clip plane / sphere intersections?
	BOOL DepthTestDisk();
public:

	// A default constructor
	ClipPlane();

	// Enable/Disable clipping
	void Enable();
	void Disable();

	// Enable/Disable display of the clipping plane
	void EnableDisplay();
	void DisableDisplay();

	void SetColor(GLfloat m_color[4]);

	void GetCenter(GLfloat m_center[3]);

	void GetNormal(GLfloat m_normal[3]);

	// Perform view clipping
	void Clip(GLfloat m_stereo_rot);

	// Center the clipping plane in the window viewport
	void CenterClip();

	// Sphere/plane intersection possiblities
	enum {NOT_RENDERED, RENDERED, CLIPED};

	// Does the given sphere intersect the clipping plane? Sort the given
	// sphere into one of three catagories: NOT_RENDERED, RENDERED or CLIPED.
	int SphereClipTest(GLfloat coord[], GLfloat radius,
						GLfloat scaleFactor, GLfloat c_x, GLfloat c_y, GLfloat c_z,
						GLfloat M[4][4]);

	// Render the clip plane
	void RenderClipPlane();
	void RenderClipSphere(array<atom> &atoms, int resolution, BOOL solvent,
		GLfloat scaleFactor, GLfloat c_x, GLfloat c_y, GLfloat c_z,
		GLfloat M[4][4]);

	void RenderClipSphere(int index, list< array<GLfloat> > &traj, 
		array<atom> &atoms, int resolution, BOOL solvent,
		GLfloat scaleFactor, GLfloat c_x, GLfloat c_y, GLfloat c_z,
		GLfloat M[4][4]);

	// Equation transformation 
	void SetEqn(GLfloat m_center[3], GLfloat m_normal[3], 
		BOOL m_norm = TRUE /* Normalize the normal!*/);

	// Zoom
	void StartZoom(int x);
	void Zoom(int x, int width);

	// Rotation
	void StartRotate(int x, int y);
	void Rotate(int x, int y, int width, int height);
};

#endif // __CLIP_PLANE