//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/

#include "stdafx.h"
#include "Qmol.h"
#include "CustomResDialog.h"
#include "QmolView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCustomResDialog dialog


CCustomResDialog::CCustomResDialog(CView *pView)
{
	m_pView = pView;

	//{{AFX_DATA_INIT(CCustomResDialog)
	m_atom_txt = _T("");
	m_bond_txt = _T("");
	m_auto_apply = TRUE;
	m_select_txt = _T("");
	m_spline_txt = _T("");
	m_tension_txt = _T("");
	m_ribbon_txt = _T("");
	//}}AFX_DATA_INIT
}


void CCustomResDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCustomResDialog)
	DDX_Control(pDX, IDC_RIBBON_SLIDER, m_ribbon_slider);
	DDX_Control(pDX, IDC_TENSION_SLIDER, m_tension_slider);
	DDX_Control(pDX, IDC_SPLINE_SLIDER, m_spline_slider);
	DDX_Control(pDX, IDC_SELECT_SLIDER, m_select_slider);
	DDX_Control(pDX, IDC_BOND_SLIDER, m_bond_slider);
	DDX_Control(pDX, IDC_ATOM_SLIDER, m_atom_slider);
	DDX_Text(pDX, IDC_ATOM_RES, m_atom_txt);
	DDX_Text(pDX, IDC_BOND_RES, m_bond_txt);
	DDX_Check(pDX, IDC_CHECK_APPLY, m_auto_apply);
	DDX_Text(pDX, IDC_SELECT_RES, m_select_txt);
	DDX_Text(pDX, IDC_SPLINE_RES, m_spline_txt);
	DDX_Text(pDX, IDC_SPLINE_TENSION, m_tension_txt);
	DDX_Text(pDX, IDC_RIBBON_RES, m_ribbon_txt);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CCustomResDialog, CDialog)
	//{{AFX_MSG_MAP(CCustomResDialog)
	ON_WM_HSCROLL()
	ON_BN_CLICKED(IDC_CHECK_APPLY, OnCheckApply)
	ON_BN_CLICKED(IDAPPLY, OnApply)
	ON_BN_CLICKED(IDCANCEL, OnClose)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCustomResDialog message handlers
BOOL CCustomResDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	CQmolView *pView = (CQmolView *)m_pView;

	// If auto apply is turned on, disable the apply button.
	GetDlgItem(IDAPPLY)->EnableWindow(!m_auto_apply);

	// Set the resolution data from the view
	m_atomRes = pView->atomRes;
	m_bondRes = pView->bondRes;
	m_selectionRes = pView->selectionRes;
	m_splineRes = pView->splineRes;
	m_splineTension = (int)(SPLINE_TENSION_NORM*
		(pView->splineTension - SPLINE_TENSION_MIN)/
		(SPLINE_TENSION_MAX - SPLINE_TENSION_MIN));

	m_ribbonRes = pView->ribbonRes;

	// Initialize the text display
	m_atom_txt.Format("%d", m_atomRes);
	m_bond_txt.Format("%d", m_bondRes);
	m_select_txt.Format("%d", m_selectionRes);
	m_spline_txt.Format("%d", m_splineRes);
	m_tension_txt.Format("%.3f", GetSplineTension(m_splineTension));
	m_ribbon_txt.Format("%d", m_ribbonRes);

	// Initialize the sliders (with hard coded limits - bad bad bad bad).
	m_atom_slider.SetRange(2, 75);
	m_atom_slider.SetPos(m_atomRes);

	m_bond_slider.SetRange(2, 75);
	m_bond_slider.SetPos(m_bondRes);

	m_select_slider.SetRange(3, 75);
	m_select_slider.SetPos(m_selectionRes);

	m_spline_slider.SetRange(0, 50);
	m_spline_slider.SetPos(m_splineRes);

	m_tension_slider.SetRange(0, SPLINE_TENSION_NORM);
	m_tension_slider.SetPos(m_splineTension);

	m_ribbon_slider.SetRange(3, 50);
	m_ribbon_slider.SetPos(m_ribbonRes);

	UpdateData(FALSE);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

BOOL CCustomResDialog::Create()
{
	return CDialog::Create(CCustomResDialog::IDD);
}

void CCustomResDialog::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	if(pScrollBar){
		// Which scroll bar did the user change?
		if((CSliderCtrl*)pScrollBar == &m_atom_slider){
			m_atomRes = m_atom_slider.GetPos();

			m_atom_txt.Format("%d", m_atomRes);

			if(m_auto_apply){
				// Notify the view that the resolution has changed
				UpdateRes(UPDATE_ATOM);
			}

			UpdateData(FALSE);

			CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
			return;
		}

		if((CSliderCtrl*)pScrollBar == &m_bond_slider){
			m_bondRes = m_bond_slider.GetPos();
			
			m_bond_txt.Format("%d", m_bondRes);

			if(m_auto_apply){
				// Notify the view that the resolution has changed
				UpdateRes(UPDATE_BOND);
			}

			UpdateData(FALSE);

			CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
			return;
		}

		if((CSliderCtrl*)pScrollBar == &m_select_slider){
			m_selectionRes = m_select_slider.GetPos();

			m_select_txt.Format("%d", m_selectionRes);

			if(m_auto_apply){
				// Notify the view that the resolution has changed
				UpdateRes(UPDATE_SELECT);
			}
			
			UpdateData(FALSE);

			CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
			return;
		}

		if((CSliderCtrl*)pScrollBar == &m_spline_slider){
			m_splineRes = m_spline_slider.GetPos();

			m_spline_txt.Format("%d", m_splineRes);

			if(m_auto_apply){
				// Notify the view that the resolution has changed
				UpdateRes(UPDATE_SPLINE);
			}
			
			UpdateData(FALSE);

			CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
			return;
		}

		if((CSliderCtrl*)pScrollBar == &m_tension_slider){
			m_splineTension = m_tension_slider.GetPos();

			m_tension_txt.Format("%.3f", GetSplineTension(m_splineTension));

			if(m_auto_apply){
				// Notify the view that the resolution has changed
				UpdateRes(UPDATE_TENSION);
			}
			
			UpdateData(FALSE);

			CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
			return;
		}

		if((CSliderCtrl*)pScrollBar == &m_ribbon_slider){
			m_ribbonRes = m_ribbon_slider.GetPos();

			m_ribbon_txt.Format("%d", m_ribbonRes);

			if(m_auto_apply){
				// Notify the view that the resolution has changed
				UpdateRes(UPDATE_RIBBON);
			}
			
			UpdateData(FALSE);

			CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
			return;
		}

		throw QMOL_ERROR("Bad scroll bar pointer!");
	}

	CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
}

void CCustomResDialog::OnCheckApply() 
{
	// Toggle the auto apply flag
	m_auto_apply = !m_auto_apply;

	GetDlgItem(IDAPPLY)->EnableWindow(!m_auto_apply);
}

void CCustomResDialog::OnApply() 
{
	// Update all of the values
	UpdateRes(UPDATE_ATOM);
	UpdateRes(UPDATE_BOND);
	UpdateRes(UPDATE_SELECT);
	UpdateRes(UPDATE_SPLINE);
	UpdateRes(UPDATE_TENSION);
	UpdateRes(UPDATE_RIBBON);
}

void CCustomResDialog::UpdateRes(int type)
{
	switch(type){
		case UPDATE_ATOM:
			m_pView->PostMessage(WM_RESOLUTION, UPDATE_ATOM, m_atomRes);
			break;

		case UPDATE_BOND:
			m_pView->PostMessage(WM_RESOLUTION, UPDATE_BOND, m_bondRes);
			break;

		case UPDATE_SELECT:
			m_pView->PostMessage(WM_RESOLUTION, UPDATE_SELECT, m_selectionRes);
			break;

		case UPDATE_SPLINE:
			m_pView->PostMessage(WM_RESOLUTION, UPDATE_SPLINE, m_splineRes);
			break;

		case UPDATE_TENSION:
			m_pView->PostMessage(WM_RESOLUTION, UPDATE_TENSION, m_splineTension);
			break;

		case UPDATE_RIBBON:
			m_pView->PostMessage(WM_RESOLUTION, UPDATE_RIBBON, m_ribbonRes);
			break;
	};
}

void CCustomResDialog::OnClose() 
{
	// Modeless case -- don't call the base class OnCancel
	m_pView->PostMessage(WM_RESOLUTION, CLOSE_DLG);
}
