//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/
// 
#ifndef EXCEPTS_H__
#define EXCEPTS_H__

// An exception class for the QMOL program

class QmolError     { 
private: 
	CString msg; 
	CString filename; 
	int linenum;

public:	
	QmolError(const char *m, char *fn = NULL, int ln = 0) : 
		msg(m), filename(fn), linenum(ln)
	{
	};

	QmolError(CString &m, char *fn = NULL, int ln = 0) : 
		msg(m), filename(fn), linenum(ln)
	{
		// Nothing to do here.
	};

	// Return error message 
	CString message()
	{
		if (!msg.IsEmpty())
			return msg; 
		
		return CString("Excepts error thrown");
	};

	int line()
	{
		return linenum;
	};

	CString file()
	{
		if(filename.IsEmpty())
			return filename;
		
		return CString("Unknown filename!");
	};
	
};

// A handy macro to pass the filename and line number to
// the error class
#define		QMOL_ERROR(string)	QmolError(string, __FILE__, __LINE__)

#endif 
