//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/

#include "stdafx.h"
#include "Qmol.h"
#include "ExportTrajDialog.h"
#include "QmolDoc.h" // Needed for file type definitions

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CExportTrajDialog dialog


CExportTrajDialog::CExportTrajDialog(int m_min_step, int m_max_step)
	: CDialog(CExportTrajDialog::IDD, NULL)
{
	render_all = TRUE;

	limit_traj_min = m_min_step;
	limit_traj_max = m_max_step;

	//{{AFX_DATA_INIT(CExportTrajDialog)
	m_file_type = QM_PDB;
	m_file_name = _T("");
	m_start_traj = m_min_step;
	m_stop_traj = m_max_step;
	//}}AFX_DATA_INIT
}


void CExportTrajDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CExportTrajDialog)
	DDX_CBIndex(pDX, IDC_COMBO_TYPE, m_file_type);
	DDX_Text(pDX, IDC_EDIT_NAME, m_file_name);
	DDX_Text(pDX, IDC_EDIT_START, m_start_traj);
	DDX_Text(pDX, IDC_EDIT_STOP, m_stop_traj);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CExportTrajDialog, CDialog)
	//{{AFX_MSG_MAP(CExportTrajDialog)
	ON_BN_CLICKED(IDC_RADIO_ALL, OnRadioAll)
	ON_BN_CLICKED(IDC_RADIO_RANGE, OnRadioRange)
	ON_BN_CLICKED(IDC_BROWSE, OnBrowse)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CExportTrajDialog message handlers

BOOL CExportTrajDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the radio buttons
	if(render_all){
		CheckRadioButton(IDC_RADIO_ALL,		// First button
						 IDC_RADIO_RANGE,	// Last bottom	
						 IDC_RADIO_ALL);

		OnRadioAll();
	}
	else{
		CheckRadioButton(IDC_RADIO_ALL,		// First button
						 IDC_RADIO_RANGE,	// Last bottom	
						 IDC_RADIO_RANGE);

		OnRadioRange();
	}

	return TRUE;
}

void CExportTrajDialog::OnRadioAll() 
{
	render_all = TRUE;
	
	GetDlgItem(IDC_EDIT_START)->EnableWindow(FALSE);
	GetDlgItem(IDC_EDIT_STOP)->EnableWindow(FALSE);
}

void CExportTrajDialog::OnRadioRange() 
{
	render_all = FALSE;

	GetDlgItem(IDC_EDIT_START)->EnableWindow(TRUE);
	GetDlgItem(IDC_EDIT_STOP)->EnableWindow(TRUE);
	
}

void CExportTrajDialog::OnBrowse() 
{
	// Set the current directory
	char dirBuffer[1024];

	switch(m_file_type){
		case QM_PDB:
			{
				CFileDialog fDlg(FALSE /* Save As*/ , "*.pdb", m_file_name);

				fDlg.m_ofn.lpstrTitle = "Specify PDB file to create";
				fDlg.m_ofn.lpstrFilter = "PDB files\0*.pdb\0\0";
				fDlg.m_ofn.Flags |= OFN_OVERWRITEPROMPT;

				// If the buffer is not big enought, DON'T set the current 
				// directory
				if(GetCurrentDirectory(1023, dirBuffer) != 0)
					fDlg.m_ofn.lpstrInitialDir = dirBuffer;

				if(fDlg.DoModal() == IDOK){
					m_file_name = fDlg.GetPathName();

					UpdateData(FALSE);
				}
			}
			break;
	};
}

void CExportTrajDialog::OnOK() 
{
	UpdateData(TRUE);
	
	if(m_file_name.GetLength() == 0){
		AfxMessageBox("Please enter a file name.");
		return;
	}

	if(render_all){
		m_start_traj = limit_traj_min;
		m_stop_traj = limit_traj_max;
	}
	else{
		// Make sure the trajectory limits have been observed
		if(m_start_traj < limit_traj_min){
			AfxMessageBox("Invalid limit for trajectory start");
			return;
		}

		if(m_stop_traj > limit_traj_max){
			AfxMessageBox("Invalid limit for trajectory stop");
			return;
		}
	}

	CDialog::OnOK();
}
