//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/
// 
#include "stdafx.h"
#include "Qmol.h"
#include "FlexColorDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CFlexColorDialog dialog


CFlexColorDialog::CFlexColorDialog(CWnd* pParent, 
								   BOOL m_show_alpha/*= FALSE*/)
	: CDialog(CFlexColorDialog::IDD, pParent)
{
	m_pWnd = pParent;

	m_msg_tag = 0;

	// Default color is opaque white
	r = g = b = a = (unsigned char)(255);
	tmp_r = tmp_g = tmp_b = tmp_a = (unsigned char)(255);

	show_alpha = m_show_alpha;

	// The default font; 12 pt Arial
	font.lfHeight = -12;
	font.lfWidth = 0;
	font.lfEscapement = 900; // 90 degree rotation
	font.lfOrientation = font.lfEscapement;
	font.lfWeight = FW_NORMAL;
	font.lfItalic = FALSE;
	font.lfUnderline = FALSE;
	font.lfStrikeOut = FALSE;
	font.lfCharSet = ANSI_CHARSET;
	font.lfOutPrecision	= OUT_DEFAULT_PRECIS;
	font.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	font.lfQuality = DEFAULT_QUALITY;
	font.lfPitchAndFamily = FF_DONTCARE|DEFAULT_PITCH;
	lstrcpy ( font.lfFaceName, "Arial" );

	// Create the font
	tmp_font.CreateFontIndirect(&font);

	// What text will we display in the color rectangle?
	text = "Transparency";

	//{{AFX_DATA_INIT(CFlexColorDialog)
	m_auto_apply = TRUE;
	//}}AFX_DATA_INIT
}

void CFlexColorDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CFlexColorDialog)
	DDX_Control(pDX, IDC_SLIDER_R, m_slide_r);
	DDX_Control(pDX, IDC_SLIDER_G, m_slide_g);
	DDX_Control(pDX, IDC_SLIDER_B, m_slide_b);
	DDX_Control(pDX, IDC_SLIDER_A, m_slide_a);
	DDX_Control(pDX, IDC_EDIT_COLOR_R, m_text_r);
	DDX_Control(pDX, IDC_EDIT_COLOR_G, m_text_g);
	DDX_Control(pDX, IDC_EDIT_COLOR_B, m_text_b);
	DDX_Control(pDX, IDC_EDIT_COLOR_A, m_text_a);
	DDX_Check(pDX, IDC_AUTO_APPLY, m_auto_apply);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CFlexColorDialog, CDialog)
	//{{AFX_MSG_MAP(CFlexColorDialog)
	ON_WM_VSCROLL()
	ON_EN_CHANGE(IDC_EDIT_COLOR_B, OnChangeEditColorB)
	ON_EN_CHANGE(IDC_EDIT_COLOR_G, OnChangeEditColorG)
	ON_EN_CHANGE(IDC_EDIT_COLOR_R, OnChangeEditColorR)
	ON_WM_PAINT()
	ON_BN_CLICKED(IDAPPLY, OnApply)
	ON_EN_CHANGE(IDC_EDIT_COLOR_A, OnChangeEditColorA)
	ON_BN_CLICKED(IDC_AUTO_APPLY, OnAutoApply)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFlexColorDialog message handlers
BOOL CFlexColorDialog::Create(int msg_tag)
{
	// Reply to the calling thread with a message containing the
	// LPARAM field m_msg_tag.
	m_msg_tag = msg_tag;

	return CDialog::Create(CFlexColorDialog::IDD);
}

BOOL CFlexColorDialog::Create()
{
	// Reply to the calling thread with a message containing the
	// LPARAM field set to zero.
	m_msg_tag = 0;

	return CDialog::Create(CFlexColorDialog::IDD);
}


BOOL CFlexColorDialog::OnInitDialog()
{
	CDialog::OnInitDialog();
	
	m_slide_r.SetRange(0, 255);
	m_slide_g.SetRange(0, 255);
	m_slide_b.SetRange(0, 255);
	m_slide_a.SetRange(0, 255);

	m_slide_r.SetPos(255 - r);
	m_slide_g.SetPos(255 - g);
	m_slide_b.SetPos(255 - b);
	m_slide_a.SetPos(255 - a);

	CString buffer;

	buffer.Format("%d", r);
	m_text_r.SetWindowText(buffer);

	buffer.Format("%d", g);
	m_text_g.SetWindowText(buffer);

	buffer.Format("%d", b);
	m_text_b.SetWindowText(buffer);

	buffer.Format("%d", a);
	m_text_a.SetWindowText(buffer);

	// Dynamically creat a painted rectangle in the dialog
	// The placeholder is a static rectangle that
	// lets us easily size the painted area.
	GetDlgItem(IDC_COLOR_STRIP)->GetWindowRect(&PaintRect);
	ScreenToClient(&PaintRect);

	tmp_r = r;
	tmp_g = g;
	tmp_b = b;
	tmp_a = a;

	if(!show_alpha){
		// Deactive the alpha selector
		GetDlgItem(IDC_SLIDER_A)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_COLOR_A)->EnableWindow(FALSE);
	}

	if(m_auto_apply){
		// Deactivate the apply button
		GetDlgItem(IDAPPLY)->EnableWindow(FALSE);
	}
	
	UpdateData();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CFlexColorDialog::GetRGB(int &m_r, int &m_g, int &m_b)
{
	m_r = r;
	m_g = g;
	m_b = b;
}

void CFlexColorDialog::GetRGBA(int &m_r, int &m_g, int &m_b, int &m_a)
{
	m_r = r;
	m_g = g;
	m_b = b;
	m_a = a;
}

void CFlexColorDialog::SetRGB(int m_r, int m_g, int m_b)
{
	// Don't allow the user to adjust the alpha channel
	show_alpha = FALSE;

	// Only accept valid 24 bit colors
	r = abs(m_r)%256;
	g = abs(m_g)%256;
	b = abs(m_b)%256;
	a = 255;

	tmp_r = r;
	tmp_g = g;
	tmp_b = b;
	tmp_a = a;

	if(GetSafeHwnd()){
		m_slide_r.SetPos(255 - r);
		m_slide_g.SetPos(255 - g);
		m_slide_b.SetPos(255 - b);

		// Lock the alpha slider is
		// the "opaque" position.
		m_slide_a.SetPos(255- a);

		CString buffer;

		buffer.Format("%d", r);
		m_text_r.SetWindowText(buffer);

		buffer.Format("%d", g);
		m_text_g.SetWindowText(buffer);

		buffer.Format("%d", b);
		m_text_b.SetWindowText(buffer);
	}
}

void CFlexColorDialog::SetRGBA(int m_r, int m_g, int m_b, int m_a)
{
	// Allow the user to adjust the alpha channel
	show_alpha = TRUE;

	// Only accept valid 24 bit colors
	r = abs(m_r)%256;
	g = abs(m_g)%256;
	b = abs(m_b)%256;
	a = abs(m_a)%256;

	tmp_r = r;
	tmp_g = g;
	tmp_b = b;
	tmp_a = a;

	if(GetSafeHwnd()){
		m_slide_r.SetPos(255 - r);
		m_slide_g.SetPos(255 - g);
		m_slide_b.SetPos(255 - b);
		m_slide_a.SetPos(255 - a);

		CString buffer;

		buffer.Format("%d", r);
		m_text_r.SetWindowText(buffer);

		buffer.Format("%d", g);
		m_text_g.SetWindowText(buffer);

		buffer.Format("%d", b);
		m_text_b.SetWindowText(buffer);

		buffer.Format("%d", a);
		m_text_a.SetWindowText(buffer);
	}
}

void CFlexColorDialog::UpdateColor(CDC *pDC /* = NULL */)
{
	blend = (float)(tmp_a)/255.0f;
	one_minus_blend = 1.0f - blend;

	clr_ref = tmp_r + (tmp_g << 8) + (tmp_b << 16);
	
	// If blend == 1, text_clr == clr_ref.
	// If blend == 0, text_clr == inverse of clr_ref
	unsigned char i_r = 255, i_g = 255, i_b = 255;

	if((tmp_r < 100) || (tmp_r > 150))
		i_r -= tmp_r;

	if((tmp_g < 100) || (tmp_g > 150))
		i_g -= tmp_g;

	if((tmp_b < 100) || (tmp_b > 150))
		i_b -= tmp_b;

	text_clr = (unsigned char)(blend*tmp_r + one_minus_blend*i_r) + 
		((unsigned char)(blend*tmp_g + one_minus_blend*i_g) << 8) +
		((unsigned char)(blend*tmp_b + one_minus_blend*i_b) << 16);

	// Repaint the color strip
	CBrush paintBrush(clr_ref);

	if(pDC == NULL){
		pDC = GetDC();
		save_bkg_mode = pDC->SetBkMode(TRANSPARENT);
		save_text_clr = pDC->SetTextColor(text_clr);
		save_font = pDC->SelectObject(&tmp_font);
		label_size = pDC->GetTextExtent(text);

		// Paint the rectange the given color
		save_brush = pDC->SelectObject(&paintBrush);
		pDC->Rectangle(PaintRect);

		// Paint the alpha text
		pDC->TextOut(PaintRect.left + PaintRect.Width()/2 - label_size.cy/2, 
			PaintRect.bottom - PaintRect.Height()/2 + label_size.cx/2, text);

		pDC->SetBkMode(save_bkg_mode);
		pDC->SetTextColor(save_text_clr);
		pDC->SelectObject(save_font);
		pDC->SelectObject(save_brush);
		ReleaseDC(pDC);
	}
	else{
		save_bkg_mode = pDC->SetBkMode(TRANSPARENT);
		save_text_clr = pDC->SetTextColor(text_clr);
		save_font = pDC->SelectObject(&tmp_font);
		label_size = pDC->GetTextExtent(text);

		// Paint the rectange the given color
		save_brush = pDC->SelectObject(&paintBrush);
		pDC->Rectangle(PaintRect);

		// Paint the alpha text
		pDC->TextOut(PaintRect.left + PaintRect.Width()/2 - label_size.cy/2, 
			PaintRect.bottom - PaintRect.Height()/2 + label_size.cx/2, text);

		pDC->SetBkMode(save_bkg_mode);
		pDC->SetTextColor(save_text_clr);
		pDC->SelectObject(save_font);
		pDC->SelectObject(save_brush);
	}

	if(m_auto_apply)
		OnApply();
}

void CFlexColorDialog::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	tmp_r = 255 - m_slide_r.GetPos();
	tmp_g = 255 - m_slide_g.GetPos();
	tmp_b = 255 - m_slide_b.GetPos();
	tmp_a = 255 - m_slide_a.GetPos();
	
	CString buffer;

	buffer.Format("%d", tmp_r);
	m_text_r.SetWindowText(buffer);

	buffer.Format("%d", tmp_g);
	m_text_g.SetWindowText(buffer);

	buffer.Format("%d", tmp_b);
	m_text_b.SetWindowText(buffer);

	buffer.Format("%d", tmp_a);
	m_text_a.SetWindowText(buffer);

	UpdateColor();

	CDialog::OnVScroll(nSBCode, nPos, pScrollBar);
}

void CFlexColorDialog::OnChangeEditColorB() 
{
	CString buffer;

	m_text_b.GetWindowText(buffer);

	int tmp = atoi(LPCSTR(buffer));

	if((tmp < 0) || (tmp > 255)){
		buffer.Format("%d", b);
		m_text_b.SetWindowText(buffer);
	}
	else{
		tmp_b =(unsigned char)(tmp);
		m_slide_b.SetPos(255 - tmp_b);

		UpdateColor();
	}	
}

void CFlexColorDialog::OnChangeEditColorG() 
{
	CString buffer;

	m_text_g.GetWindowText(buffer);

	int tmp = atoi(LPCSTR(buffer));

	if((tmp < 0) || (tmp > 255)){
		buffer.Format("%d", tmp_g);
		m_text_g.SetWindowText(buffer);
	}
	else{
		tmp_g =(unsigned char)(tmp);
		m_slide_g.SetPos(255 - tmp_g);

		UpdateColor();
	}
}

void CFlexColorDialog::OnChangeEditColorR() 
{
	CString buffer;

	m_text_r.GetWindowText(buffer);

	int tmp = atoi(LPCSTR(buffer));

	if((tmp < 0) || (tmp > 255)){
		buffer.Format("%d", tmp_r);
		m_text_r.SetWindowText(buffer);
	}
	else{
		tmp_r =(unsigned char)(tmp);
		m_slide_r.SetPos(255 - tmp_r);

		UpdateColor();
	}
}

void CFlexColorDialog::OnCancel() 
{
	if(m_pWnd != NULL){
		// Modeless case -- don't call the base class OnCancel
		m_pWnd->PostMessage(WM_FLEX_COLOR, IDCANCEL, m_msg_tag);
	}
}

void CFlexColorDialog::OnOK() 
{
	if(m_pWnd != NULL){
		r = tmp_r;
		g = tmp_g;
		b = tmp_b;
		a = tmp_a;

		// Modeless case -- don't call the base class OnCancel
		m_pWnd->PostMessage(WM_FLEX_COLOR, IDOK, m_msg_tag);
	}
}

void CFlexColorDialog::OnApply() 
{
	if(m_pWnd != NULL){
		r = tmp_r;
		g = tmp_g;
		b = tmp_b;
		a = tmp_a;

		// Modeless case -- don't call the base class OnCancel
		m_pWnd->PostMessage(WM_FLEX_COLOR, FLEX_COLOR_APPLY, m_msg_tag);
	}
}

void CFlexColorDialog::OnPaint() 
{
	// device context for painting. UpdateColor has
	// been overloaded to use this device context. If we 
	// attempt to use GetDC(), the message cue gets jamed for the
	// entire program!
	CPaintDC dc(this); 
	
	UpdateColor(&dc);
}

void CFlexColorDialog::OnChangeEditColorA() 
{
	CString buffer;

	m_text_a.GetWindowText(buffer);

	int tmp = atoi(LPCSTR(buffer));

	if((tmp < 0) || (tmp > 255)){
		buffer.Format("%d", tmp_a);
		m_text_a.SetWindowText(buffer);
	}
	else{
		tmp_a =(unsigned char)(tmp);
		m_slide_a.SetPos(255 - tmp_a);

		UpdateColor();
	}
	
}

void CFlexColorDialog::OnAutoApply() 
{
	if(m_auto_apply){
		m_auto_apply = FALSE;
		GetDlgItem(IDAPPLY)->EnableWindow(TRUE);
	}
	else{
		m_auto_apply = TRUE;
		GetDlgItem(IDAPPLY)->EnableWindow(FALSE);
	}
}
