//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/
//

#include "stdafx.h"
#include "Qmol.h"
#include "LightDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CLightDialog dialog


CLightDialog::CLightDialog(CWnd* pParent)
	: CDialog(CLightDialog::IDD, pParent)
{
	m_pWnd = pParent;

	// Default color is opaque white
	r = g = b = (unsigned char)(255);

	// Default material properties (as percentages)
	shiny = specular = (unsigned char)(100); // i.e. 100 %

	// Set up the color dialog
	flexCDlg = new CFlexColorDialog(this);

	//{{AFX_DATA_INIT(CLightDialog)
	m_auto_apply = TRUE;
	m_back = FALSE;
	m_bottom = FALSE;
	m_front = FALSE;
	m_left = FALSE;
	m_right = FALSE;
	m_top = FALSE;
	m_ambient = FALSE;
	m_shiny_text = _T("");
	m_specular_text = _T("");
	//}}AFX_DATA_INIT
}

CLightDialog::~CLightDialog()
{
	if(flexCDlg)
		delete flexCDlg;
}

void CLightDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CLightDialog)
	DDX_Control(pDX, IDC_SPECULAR_SLIDER, m_specular_slider);
	DDX_Control(pDX, IDC_SHINY_SLIDER, m_shiny_slider);
	DDX_Control(pDX, IDC_R_EDIT, m_r_text);
	DDX_Control(pDX, IDC_G_EDIT, m_g_text);
	DDX_Control(pDX, IDC_B_EDIT, m_b_text);
	DDX_Check(pDX, IDC_AUTO_APPLY_CHECK, m_auto_apply);
	DDX_Check(pDX, IDC_BACK_CHECK, m_back);
	DDX_Check(pDX, IDC_BOTTOM_CHECK, m_bottom);
	DDX_Check(pDX, IDC_FRONT_CHECK, m_front);
	DDX_Check(pDX, IDC_LEFT_CHECK, m_left);
	DDX_Check(pDX, IDC_RIGHT_CHECK, m_right);
	DDX_Check(pDX, IDC_TOP_CHECK, m_top);
	DDX_Check(pDX, IDC_AMBIENT_CHECK, m_ambient);
	DDX_Text(pDX, IDC_SHINY_TEXT, m_shiny_text);
	DDX_Text(pDX, IDC_SPECULAR_TEXT, m_specular_text);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CLightDialog, CDialog)
	//{{AFX_MSG_MAP(CLightDialog)
	ON_BN_CLICKED(IDAPPLY, OnApply)
	ON_BN_CLICKED(IDC_AUTO_APPLY_CHECK, OnAutoApplyCheck)
	ON_EN_CHANGE(IDC_B_EDIT, OnChangeBEdit)
	ON_BN_CLICKED(IDC_BACK_CHECK, OnBackCheck)
	ON_BN_CLICKED(IDC_BOTTOM_CHECK, OnBottomCheck)
	ON_BN_CLICKED(IDC_FRONT_CHECK, OnFrontCheck)
	ON_EN_CHANGE(IDC_G_EDIT, OnChangeGEdit)
	ON_BN_CLICKED(IDC_LEFT_CHECK, OnLeftCheck)
	ON_EN_CHANGE(IDC_R_EDIT, OnChangeREdit)
	ON_BN_CLICKED(IDC_RIGHT_CHECK, OnRightCheck)
	ON_BN_CLICKED(IDC_TOP_CHECK, OnTopCheck)
	ON_BN_CLICKED(IDC_COLOR_BUTTON, OnColorButton)
	ON_MESSAGE(WM_FLEX_COLOR, OnFlexColorMsg)
	ON_BN_CLICKED(IDC_AMBIENT_CHECK, OnAmbientCheck)
	ON_WM_HSCROLL()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLightDialog message handlers

BOOL CLightDialog::Create()
{
	return CDialog::Create(CLightDialog::IDD);
}

BOOL CLightDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	CString buffer;

	buffer.Format("%d", r);
	m_r_text.SetWindowText(buffer);

	buffer.Format("%d", g);
	m_g_text.SetWindowText(buffer);

	buffer.Format("%d", b);
	m_b_text.SetWindowText(buffer);

	// Initialize the material property sliders
	m_shiny_slider.SetRange(0, 100);
	m_specular_slider.SetRange(0, 100);

	m_shiny_slider.SetPos(shiny);
	m_specular_slider.SetPos(specular);

	m_shiny_text.Format("%d%% shiny", shiny);
	m_specular_text.Format("%d%% specular", specular);

	if(m_auto_apply){
		// Deactivate the apply button
		GetDlgItem(IDAPPLY)->EnableWindow(FALSE);
	}
	
	UpdateData(FALSE);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CLightDialog::SetRGB(int m_r, int m_g, int m_b)
{
	r = m_r;
	g = m_g;
	b = m_b;

	if(GetSafeHwnd()){
		CString buffer;

		buffer.Format("%d", r);
		m_r_text.SetWindowText(buffer);

		buffer.Format("%d", g);
		m_g_text.SetWindowText(buffer);

		buffer.Format("%d", b);
		m_b_text.SetWindowText(buffer);
	}
}

void CLightDialog::GetRGB(int &m_r, int &m_g, int &m_b)
{
	m_r = r;
	m_g = g;
	m_b = b;
}

void CLightDialog::SetMaterialProperties(int m_shiny, int m_specular)
{
	shiny = m_shiny;
	specular = m_specular;

	if(GetSafeHwnd()){

		m_shiny_text.Format("%d%% shiny", shiny);
		m_specular_text.Format("%d%% specular", specular);

		UpdateData(FALSE);
	}
}

void CLightDialog::GetMaterialProperties(int &m_shiny, int &m_specular)
{
	m_shiny = shiny;
	m_specular = specular;
}


void CLightDialog::SetLightStatus(LightDef m_l, BOOL m_status)
{
	switch(m_l){
		case FRONT:
			m_front = m_status;
			break;
		case BACK:
			m_back = m_status;
			break;
		case TOP:
			m_top = m_status;
			break;
		case BOTTOM:
			m_bottom = m_status;
			break;
		case LEFT:
			m_left = m_status;
			break;
		case RIGHT:
			m_right = m_status;
			break;
		case AMBIENT:
			m_ambient = m_status;
			break;
	};

	if(GetSafeHwnd())
		UpdateData(FALSE);
}

BOOL CLightDialog::GetLightStatus(LightDef m_l)
{
	switch(m_l){
		case FRONT:
			return m_front;
		case BACK:
			return m_back;
		case TOP:
			return m_top;
		case BOTTOM:
			return m_bottom;
		case LEFT:
			return m_left;
		case RIGHT:
			return m_right;
	};

	// case AMBIENT:
	return m_ambient;
}

void CLightDialog::OnApply() 
{
	if(m_pWnd != NULL)
		m_pWnd->PostMessage(WM_LIGHTING, IDAPPLY);
}

void CLightDialog::OnAutoApplyCheck() 
{
	if(m_auto_apply){
		m_auto_apply = FALSE;
		GetDlgItem(IDAPPLY)->EnableWindow(TRUE);
	}
	else{
		m_auto_apply = TRUE;
		GetDlgItem(IDAPPLY)->EnableWindow(FALSE);
	}
}

void CLightDialog::OnBackCheck() 
{
	m_back = !m_back;

	if(m_auto_apply)
		OnApply();
}

void CLightDialog::OnBottomCheck() 
{
	m_bottom = !m_bottom;

	if(m_auto_apply)
		OnApply();
}

void CLightDialog::OnFrontCheck() 
{
	m_front = !m_front;

	if(m_auto_apply)
		OnApply();
}

void CLightDialog::OnLeftCheck() 
{
	m_left = !m_left;

	if(m_auto_apply)
		OnApply();
}

void CLightDialog::OnRightCheck() 
{
	m_right = !m_right;

	if(m_auto_apply)
		OnApply();
}

void CLightDialog::OnTopCheck() 
{
	m_top = !m_top;

	if(m_auto_apply)
		OnApply();
}

void CLightDialog::OnAmbientCheck() 
{
	m_ambient = !m_ambient;

	if(m_auto_apply)
		OnApply();
}

void CLightDialog::OnOK() 
{
	if(m_pWnd != NULL){
		if(flexCDlg->GetSafeHwnd())
			flexCDlg->DestroyWindow();

		m_pWnd->PostMessage(WM_LIGHTING, IDOK);
	}
}

void CLightDialog::OnCancel() 
{
	if(m_pWnd != NULL){
		if(flexCDlg->GetSafeHwnd())
			flexCDlg->DestroyWindow();

		m_pWnd->PostMessage(WM_LIGHTING, IDCANCEL);
	}
}

void CLightDialog::OnChangeREdit() 
{
	CString buffer;

	m_r_text.GetWindowText(buffer);

	int tmp = atoi(LPCSTR(buffer));

	if((tmp < 0) || (tmp > 255)){
		buffer.Format("%d", r);
		m_r_text.SetWindowText(buffer);
	}
	else{
		r = tmp;
	}

	if(m_auto_apply)
		OnApply();
}

void CLightDialog::OnChangeGEdit() 
{
	CString buffer;

	m_g_text.GetWindowText(buffer);

	int tmp = atoi(LPCSTR(buffer));

	if((tmp < 0) || (tmp > 255)){
		buffer.Format("%d", g);
		m_g_text.SetWindowText(buffer);
	}
	else{
		g = tmp;
	}

	if(m_auto_apply)
		OnApply();
}

void CLightDialog::OnChangeBEdit() 
{
	CString buffer;

	m_b_text.GetWindowText(buffer);

	int tmp = atoi(LPCSTR(buffer));

	if((tmp < 0) || (tmp > 255)){
		buffer.Format("%d", b);
		m_b_text.SetWindowText(buffer);
	}
	else{
		b = tmp;
	}

	if(m_auto_apply)
		OnApply();
}

void CLightDialog::OnColorButton() 
{
	if(flexCDlg->GetSafeHwnd() == 0){
		flexCDlg->SetRGB(r, g, b);

		flexCDlg->Create();
	}
}

LRESULT CLightDialog::OnFlexColorMsg(WPARAM wParam, LPARAM lParam) 
{
	int tmp_r, tmp_g, tmp_b;

	CString buffer;

	switch(wParam){
		case IDOK:			
			flexCDlg->GetRGB(tmp_r, tmp_g, tmp_b);

			r = tmp_r;
			g = tmp_g;
			b = tmp_b;

			buffer.Format("%d", r);
			m_r_text.SetWindowText(buffer);

			buffer.Format("%d", g);
			m_g_text.SetWindowText(buffer);

			buffer.Format("%d", b);
			m_b_text.SetWindowText(buffer);

			flexCDlg->DestroyWindow();
			break;
		case IDCANCEL:
			flexCDlg->DestroyWindow();
			break;
		case FLEX_COLOR_APPLY:
			flexCDlg->GetRGB(tmp_r, tmp_g, tmp_b);
			
			r = tmp_r;
			g = tmp_g;
			b = tmp_b;

			buffer.Format("%d", r);
			m_r_text.SetWindowText(buffer);

			buffer.Format("%d", g);
			m_g_text.SetWindowText(buffer);

			buffer.Format("%d", b);
			m_b_text.SetWindowText(buffer);
	};

	return 0L;
}


void CLightDialog::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	shiny = m_shiny_slider.GetPos();
	specular = m_specular_slider.GetPos();

	m_shiny_text.Format("%d%% shiny", shiny);
	m_specular_text.Format("%d%% specular", specular);

	UpdateData(FALSE);

	if(m_auto_apply){
		OnApply();
	}

	CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
}
