//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/

#include "stdafx.h"
#include "Qmol.h"
#include "QmolView.h"
#include "MeasureDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMeasureDialog dialog


CMeasureDialog::CMeasureDialog(CView* pParent)
	: CDialog(CMeasureDialog::IDD, pParent)
{
	m_pView = pParent;

	//{{AFX_DATA_INIT(CMeasureDialog)
	m_a_num = _T("");
	m_a_x = _T("");
	m_a_y = _T("");
	m_a_z = _T("");
	m_angle_abc = _T("");
	m_angle_bcd = _T("");
	m_b_num = _T("");
	m_b_x = _T("");
	m_b_y = _T("");
	m_b_z = _T("");
	m_c_num = _T("");
	m_c_x = _T("");
	m_c_y = _T("");
	m_c_z = _T("");
	m_d_num = _T("");
	m_d_x = _T("");
	m_d_y = _T("");
	m_d_z = _T("");
	m_torsion_abcd = _T("");
	m_bond_ab = _T("");
	m_bond_bc = _T("");
	m_bond_cd = _T("");
	m_a_model = _T("");
	m_b_model = _T("");
	m_c_model = _T("");
	m_d_model = _T("");
	//}}AFX_DATA_INIT
}


void CMeasureDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMeasureDialog)
	DDX_Text(pDX, IDC_MEASURE_A_NUM, m_a_num);
	DDX_Text(pDX, IDC_MEASURE_A_X, m_a_x);
	DDX_Text(pDX, IDC_MEASURE_A_Y, m_a_y);
	DDX_Text(pDX, IDC_MEASURE_A_Z, m_a_z);
	DDX_Text(pDX, IDC_MEASURE_ANGLE_ABC, m_angle_abc);
	DDX_Text(pDX, IDC_MEASURE_ANGLE_BCD, m_angle_bcd);
	DDX_Text(pDX, IDC_MEASURE_B_NUM, m_b_num);
	DDX_Text(pDX, IDC_MEASURE_B_X, m_b_x);
	DDX_Text(pDX, IDC_MEASURE_B_Y, m_b_y);
	DDX_Text(pDX, IDC_MEASURE_B_Z, m_b_z);
	DDX_Text(pDX, IDC_MEASURE_C_NUM, m_c_num);
	DDX_Text(pDX, IDC_MEASURE_C_X, m_c_x);
	DDX_Text(pDX, IDC_MEASURE_C_Y, m_c_y);
	DDX_Text(pDX, IDC_MEASURE_C_Z, m_c_z);
	DDX_Text(pDX, IDC_MEASURE_D_NUM, m_d_num);
	DDX_Text(pDX, IDC_MEASURE_D_X, m_d_x);
	DDX_Text(pDX, IDC_MEASURE_D_Y, m_d_y);
	DDX_Text(pDX, IDC_MEASURE_D_Z, m_d_z);
	DDX_Text(pDX, IDC_MEASURE_TORSION_ABCD, m_torsion_abcd);
	DDX_Text(pDX, IDC_MEAURE_BOND_AB, m_bond_ab);
	DDX_Text(pDX, IDC_MEAURE_BOND_BC, m_bond_bc);
	DDX_Text(pDX, IDC_MEAURE_BOND_CD, m_bond_cd);
	DDX_Text(pDX, IDC_MEASURE_A_MODEL, m_a_model);
	DDX_Text(pDX, IDC_MEASURE_B_MODEL, m_b_model);
	DDX_Text(pDX, IDC_MEASURE_C_MODEL, m_c_model);
	DDX_Text(pDX, IDC_MEASURE_D_MODEL, m_d_model);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CMeasureDialog, CDialog)
	//{{AFX_MSG_MAP(CMeasureDialog)
	ON_BN_CLICKED(ID_MEASURE_CLOSE, OnMeasureClose)
	ON_BN_CLICKED(ID_MEASURE_CLEAR, OnMeasureClear)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMeasureDialog message handlers

BOOL CMeasureDialog::Create() 
{
	return CDialog::Create(CMeasureDialog::IDD);
}

void CMeasureDialog::OnMeasureClose() 
{
	if(m_pView != NULL){
		m_pView->PostMessage(WM_MEASURE_CLOSE, IDCANCEL);
	}
}

void CMeasureDialog::OnMeasureClear() 
{
	CQmolDoc* pDoc = (CQmolDoc*)(m_pView->GetDocument());
	ASSERT_VALID(pDoc);

	// Throw away previously selected measurements
	pDoc->molMetric.Flush();

	// Redraw the screen
	((CQmolView*)m_pView)->postRedisplay();

	// Update the dialog
	UpdateMeasurements();
}

BOOL CMeasureDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();

	if(m_pView != NULL)
		UpdateMeasurements();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CMeasureDialog::UpdateMeasurements()
{	
	CQmolDoc* pDoc = (CQmolDoc*)(m_pView->GetDocument());
	ASSERT_VALID(pDoc);

	if(pDoc->molMetric.NumPicked() > 0){
		m_a_x.Format("%.3f", pDoc->molMetric.x_coord(0));
		m_a_y.Format("%.3f", pDoc->molMetric.y_coord(0));
		m_a_z.Format("%.3f", pDoc->molMetric.z_coord(0));
		m_a_num.Format("%d", pDoc->molMetric.Id(0));
		m_a_model.Format("%d", pDoc->molMetric.Model(0));
	}
	else{
		m_a_x = _T("");
		m_a_y = _T("");
		m_a_z = _T("");
		m_a_num = _T("");
		m_a_model = _T("");
	}
	
	if(pDoc->molMetric.NumPicked() > 1){
		m_b_x.Format("%.3f", pDoc->molMetric.x_coord(1));
		m_b_y.Format("%.3f", pDoc->molMetric.y_coord(1));
		m_b_z.Format("%.3f", pDoc->molMetric.z_coord(1));
		m_b_num.Format("%d", pDoc->molMetric.Id(1));
		m_b_model.Format("%d", pDoc->molMetric.Model(1));
	}
	else{
		m_b_x = _T("");
		m_b_y = _T("");
		m_b_z = _T("");
		m_b_num = _T("");
		m_b_model = _T("");
	}
	
	if(pDoc->molMetric.NumPicked() > 2){
		m_c_x.Format("%.3f", pDoc->molMetric.x_coord(2));
		m_c_y.Format("%.3f", pDoc->molMetric.y_coord(2));
		m_c_z.Format("%.3f", pDoc->molMetric.z_coord(2));
		m_c_num.Format("%d", pDoc->molMetric.Id(2));
		m_c_model.Format("%d", pDoc->molMetric.Model(2));
	}	
	else{
		m_c_x = _T("");
		m_c_y = _T("");
		m_c_z = _T("");
		m_c_num = _T("");
		m_c_model = _T("");
	}
		
	if(pDoc->molMetric.NumPicked() > 3){
		m_d_x.Format("%.3f", pDoc->molMetric.x_coord(3));
		m_d_y.Format("%.3f", pDoc->molMetric.y_coord(3));
		m_d_z.Format("%.3f", pDoc->molMetric.z_coord(3));
		m_d_num.Format("%d", pDoc->molMetric.Id(3));
		m_d_model.Format("%d", pDoc->molMetric.Model(3));
	}
	else{
		m_d_x = _T("");
		m_d_y = _T("");
		m_d_z = _T("");
		m_d_num = _T("");
		m_d_model = _T("");
	}
		
	if(pDoc->molMetric.NumPicked() > 1)
		m_bond_ab.Format("%.3f", pDoc->molMetric.Bond(0));
	else
		m_bond_ab = _T("");
		
	if(pDoc->molMetric.NumPicked() > 2)
		m_bond_bc.Format("%.3f", pDoc->molMetric.Bond(1));
	else
		m_bond_bc = _T("");
		
	if(pDoc->molMetric.NumPicked() > 3)
		m_bond_cd.Format("%.3f", pDoc->molMetric.Bond(2));
	else
		m_bond_cd = _T("");
		
	if(pDoc->molMetric.NumPicked() > 2)
		m_angle_abc.Format("%.3f", pDoc->molMetric.Angle(0));
	else
		m_angle_abc = _T("");
		
	if(pDoc->molMetric.NumPicked() > 3)
		m_angle_bcd.Format("%.3f", pDoc->molMetric.Angle(1));
	else
		m_angle_bcd = _T("");
		
	if(pDoc->molMetric.NumPicked() > 3)
		m_torsion_abcd.Format("%.3f", pDoc->molMetric.Torsion());
	else
		m_torsion_abcd = _T("");

	// Use the FALSE argument to indicate that we're
	// SETTING dialog variables.
	UpdateData(FALSE);
}
