//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/

#include "stdafx.h"
#include "Qmol.h"
#include "MolSelectBar.h"
#include "MainFrm.h"
#include "Excepts.h"
#include "QmolDoc.h"

BEGIN_MESSAGE_MAP(CMolSelectBar, CDialogBar)
	//{{AFX_MSG_MAP(CMolSelectBar)
	ON_MESSAGE(WM_UPDATE_SELECTION, UpdateSelection)
	ON_NOTIFY(NM_RCLICK, IDC_MOL_LIST, OnRclickSelection)
	ON_WM_SIZING()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


void CMolSelectBar::InitDock()
{

	CMainFrame *pFrame = (CMainFrame*)AfxGetApp()->m_pMainWnd;

	BOOL status = 
		CDialogBar::Create(pFrame, IDD_MOLSEL_DIALOGBAR, 
			CBRS_BOTTOM|CBRS_SIZE_DYNAMIC, 
			IDD_MOLSEL_DIALOGBAR);

	if(!status)
		throw QmolError("Unable to create docking molecule selection bar");

	// The size and location of the list
	CRect listRect;

	listRect.left = 2;
	listRect.top = 2;
	listRect.right = listRect.left + m_sizeDefault.cx - 4;
	listRect.bottom = listRect.top + m_sizeDefault.cy - 4;

	// Create the list control
	molList.Create(LVS_REPORT|LVS_NOSORTHEADER|WS_BORDER|WS_VISIBLE,
		listRect, this, IDC_MOL_LIST);

	// Delete these three lines if you don't want the toolbar to
	//  be dockable
	EnableDocking(CBRS_ALIGN_TOP|CBRS_ALIGN_BOTTOM);
	pFrame->EnableDocking(CBRS_ALIGN_ANY);
	pFrame->DockControlBar(this);

	m_initDock = TRUE;
	m_currSize = m_sizeDefault;

	// Initialize the list control
	// Initialize the sequence list  control
	// Build the columns in the parameter list control
	LVCOLUMN tmpCol;
	CDC *pDC;
	float col_width_fudge_factor = 2.0f;
	pDC = molList.GetDC();

	tmpCol.mask = LVCF_TEXT | LVCF_FMT | LVCF_WIDTH;
	tmpCol.fmt = LVCFMT_LEFT;
	tmpCol.pszText = "Molecule";
	tmpCol.cchTextMax = strlen(tmpCol.pszText);

	tmpCol.cx = (int)(col_width_fudge_factor*
		pDC->GetTextExtent(tmpCol.pszText, tmpCol.cchTextMax).cx);

	tmpCol.iSubItem = 0;	// Ignored
	tmpCol.iImage = 0;		// Ignored
	tmpCol.iOrder = 0;		// Ignored

	molList.InsertColumn(0, &tmpCol);

	tmpCol.pszText = "Status";
	tmpCol.cchTextMax = strlen(tmpCol.pszText);

	tmpCol.cx = (int)(col_width_fudge_factor*
		pDC->GetTextExtent(tmpCol.pszText, tmpCol.cchTextMax).cx);

	molList.InsertColumn(1, &tmpCol);

	tmpCol.pszText = "Atoms";
	tmpCol.cchTextMax = strlen(tmpCol.pszText);
	tmpCol.cx = (int)(col_width_fudge_factor*
		pDC->GetTextExtent(tmpCol.pszText, tmpCol.cchTextMax).cx);

	molList.InsertColumn(2, &tmpCol);

	tmpCol.pszText = "Frames";
	tmpCol.cchTextMax = strlen(tmpCol.pszText);
	tmpCol.cx = (int)(col_width_fudge_factor*
		pDC->GetTextExtent(tmpCol.pszText, tmpCol.cchTextMax).cx);

	molList.InsertColumn(3, &tmpCol);

	molList.ReleaseDC(pDC);

	UpdateSelection(0, 0);
}

void CMolSelectBar::Show()
{
	if(!m_initDock)
		throw QmolError("Unable to display uninitialized dialog bar",
			__FILE__, __LINE__);

	CMainFrame *pFrame = (CMainFrame*)AfxGetApp()->m_pMainWnd;

	UpdateSelection(0, 0);

	if(pFrame){
		pFrame->ShowControlBar(this, TRUE, FALSE);
	}
}

void CMolSelectBar::Hide()
{
	if(!m_initDock)
		throw QmolError("Unable to display uninitialized dialog bar",
			__FILE__, __LINE__);

	CMainFrame *pFrame = (CMainFrame*)AfxGetApp()->m_pMainWnd;

	if(pFrame){
		pFrame->ShowControlBar(this, FALSE, FALSE);
	}
}

void CMolSelectBar::ReSizeBar()
{
	// Resize all of the controls in the dialogbar
	molList.MoveWindow(4, 4, m_currSize.cx - 8, m_currSize.cy - 8);
}

CSize CMolSelectBar::CalcDynamicLayout( int nLength, DWORD dwMode )
{
	if(IsFloating()){
		// The frame takes care of sizing when we're floating
		ModifyStyle(WS_THICKFRAME,0);
	}    
	else{
		// We need to handle sizing when docked
		ModifyStyle(0, WS_THICKFRAME);
	}

	// Return default if it is being docked or floated
	if ((dwMode & LM_VERTDOCK) || (dwMode & LM_HORZDOCK))
	{
		if(dwMode & LM_STRETCH)
			// Not docked stretch to fit
			return  CSize((dwMode & LM_HORZ) ? 32767 : m_currSize.cx,
				(dwMode & LM_HORZ) ? m_currSize.cy : 32767);
		else{
			// Docked -- streach to fit frame window
			CRect frameRect;
			CFrameWnd* pFrame = GetDockingFrame();
			
			pFrame->GetClientRect(&frameRect);

			int dx = frameRect.right - frameRect.left;
			
			m_currSize.cx = dx;

			ReSizeBar();
			return m_currSize;
		}
	}

	
	if(dwMode & LM_MRUWIDTH)
		return m_currSize;

	// In all other cases, accept the dynamic length
	if (dwMode & LM_LENGTHY){
		
		m_currSize.cy = nLength;

		ReSizeBar();
		return m_currSize;
	}
	else{
		m_currSize.cx = nLength;

		ReSizeBar();
		return m_currSize;
	}
}


LRESULT CMolSelectBar::UpdateSelection(WPARAM wParam, LPARAM lParam)
{
	// Dialog bar has yet to be initialized
	if(!m_initDock)
		return 0L;

	// Pack the list control with sequence data
	CQmolDoc* pDoc = (CQmolDoc*)(pView->GetDocument());
	ASSERT_VALID(pDoc);

	// Remove any pre-existing items from the list control
	molList.DeleteAllItems();

	CString buffer;
	LVITEM tmpLVITEM;
		
	// Zero out the LVITEM structure
	memset(&tmpLVITEM, 0, sizeof(LVITEM));

	for(int i = 0;i < pDoc->molList.Length();i++){

		// Add the Atom name
		tmpLVITEM.mask = LVIF_TEXT;
		tmpLVITEM.iItem = i;
		tmpLVITEM.iSubItem = 0; // The item itself
		tmpLVITEM.state = 0;
		tmpLVITEM.stateMask = LVIS_SELECTED;
		
		buffer.Format("%s", pDoc->molList[i]->coordName);
		tmpLVITEM.pszText = buffer.GetBuffer(0);
		
		molList.InsertItem(&tmpLVITEM);
		
		// Add the status
		tmpLVITEM.mask = LVIF_TEXT;
		tmpLVITEM.iItem = i;
		tmpLVITEM.iSubItem = 1; // The molcule status
		tmpLVITEM.state = 0;
		tmpLVITEM.stateMask = LVIS_SELECTED;
		
		if(pDoc->molList[i]->GetDynamic()){
			if(pDoc->molList[i] == pDoc->activeMol){
				buffer.Format("Primary, Free");
			}
			else{
				buffer.Format("Free");
			}
		}
		else{
			if(pDoc->molList[i] == pDoc->activeMol){
				buffer.Format("Primary, Fixed");
			}
			else{
				buffer.Format("Fixed");
			}
		}

		tmpLVITEM.pszText = buffer.GetBuffer(0);
		
		molList.SetItem(&tmpLVITEM);
		
		// Add the number of atoms
		tmpLVITEM.mask = LVIF_TEXT;
		tmpLVITEM.iItem = i;
		tmpLVITEM.iSubItem = 2; // The number of atoms
		tmpLVITEM.state = 0;
		tmpLVITEM.stateMask = LVIS_SELECTED;
		
		buffer.Format("%d", pDoc->molList[i]->NumAtoms());
		
		tmpLVITEM.pszText = buffer.GetBuffer(0);
		
		molList.SetItem(&tmpLVITEM);
		
		// Add the number of animation frames
		tmpLVITEM.mask = LVIF_TEXT;
		tmpLVITEM.iItem = i;
		tmpLVITEM.iSubItem = 3; // The number of animation frames
		tmpLVITEM.state = 0;
		tmpLVITEM.stateMask = LVIS_SELECTED;
		
		buffer.Format("%d", pDoc->molList[i]->steps);
		
		tmpLVITEM.pszText = buffer.GetBuffer(0);
		
		molList.SetItem(&tmpLVITEM);
	}

	return 0L;
}
void CMolSelectBar::OnRclickSelection(NMHDR* pNMHDR, LRESULT* pResult)
{
	// Build a pop-up menu
	CMenu menu; 
	VERIFY(menu.LoadMenu(IDR_SELECT_MENU));

	CMenu* pPopup = menu.GetSubMenu(0); 

	ASSERT(pPopup != NULL); 

	CPoint screenPoint;
	GetCursorPos(&screenPoint);

	pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, 
		screenPoint.x, screenPoint.y, 
		AfxGetMainWnd());

	*pResult = 0;
}

BOOL CMolSelectBar::ProcessSelectionMenu(int menuID)
{
	// Get the total number of selected molecules
	POSITION pos = molList.GetFirstSelectedItemPosition();	
	int selCount = 0;
	int curSel;

	while(pos){
		molList.GetNextSelectedItem(pos);
		selCount++;
	}

	pos = molList.GetFirstSelectedItemPosition();	
	
	// There are several options we need to handle:
	// (a) No molecules selected
	// (b) One molecule selected
	// (c) several molecule selected
	if(selCount == 0){
		return FALSE;
	}

	if(selCount == 1){
		// (b) One molecule selected
		curSel = molList.GetNextSelectedItem(pos);
		
		if(menuID == IDC_SELECT_PRIMARY){
			CQmolDoc* pDoc = (CQmolDoc*)(pView->GetDocument());
			ASSERT_VALID(pDoc);

			// Is this the primary molecule?
			if(pDoc->activeMol == pDoc->molList[curSel]){
				return FALSE;
			}
		}

		return TRUE;
	}
	
			
	// (c) several molecules selected
	if(menuID == IDC_SELECT_PRIMARY)
		return FALSE;

	return TRUE;
}

// Return the first selected molecule
int CMolSelectBar::GetSelectedMol()
{
	// Get the total number of selected molecules
	POSITION pos = molList.GetFirstSelectedItemPosition();	
	
	if(!pos)
		throw QmolError("No selected molecule found!", 
			__FILE__, __LINE__);

		return molList.GetNextSelectedItem(pos);
}

void CMolSelectBar::GetSelectedMol(list<int> &m_molSel)
{
	// Get the total number of selected molecules
	POSITION pos = molList.GetFirstSelectedItemPosition();	

	while(pos)
		m_molSel.Push(molList.GetNextSelectedItem(pos));
}

void CMolSelectBar::OnSizing(UINT fwSide, LPRECT pRect)
{
	//CalcDynamicLayout(pRect->bottom - pRect->top, LM_LENGTHY);

	CFrameWnd* pFrame = GetDockingFrame();

	// Which edge is the user moving?
	switch(fwSide){
		case 1: // Left
		
			return;
		case 2: // Right
			
			return;
		case 4: // Top left
			
			return;
		case 5: // Top right
			
			return;
		case 7: // Bottom left
			
			return;
		case 8: // Bottom right
			
			return;
		case 3: // Top
			m_currSize.cy = pRect->bottom - pRect->top;
			pFrame->RecalcLayout();
			break;
		case 6: // Bottom
			m_currSize.cy = pRect->bottom - pRect->top;
			pFrame->RecalcLayout();
			break;
	};
}
