//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/

#include "stdafx.h"
#include "Qmol.h"
#include "NetworkPBDDialog.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CNetworkPBDDialog dialog


#define SERVER_STR	"%SERVER%"
#define QUERY_STR	"%PDBID%"

CNetworkPBDDialog::CNetworkPBDDialog(list<CString> &server_list, 
		list<CString> &query_list)
	: CDialog(CNetworkPBDDialog::IDD, NULL), m_server_list(server_list),
	m_query_list(query_list)
{
	//{{AFX_DATA_INIT(CNetworkPBDDialog)
	m_format = _T("");
	m_net_status = _T("");
	//}}AFX_DATA_INIT
}


void CNetworkPBDDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CNetworkPBDDialog)
	DDX_Control(pDX, IDC_SERVER_COMBO, m_server);
	DDX_Control(pDX, IDC_PDBID_COMBO, m_pdb_id);
	DDX_Text(pDX, IDC_NETFORMAT_EDIT, m_format);
	DDX_Text(pDX, IDC_NET_STATUS, m_net_status);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CNetworkPBDDialog, CDialog)
	//{{AFX_MSG_MAP(CNetworkPBDDialog)
		// NOTE: the ClassWizard will add message map macros here
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CNetworkPBDDialog message handlers

BOOL CNetworkPBDDialog::OnInitDialog()
{
	int i;

	CDialog::OnInitDialog();
	
	// Initialize the dialog, starting with the server list
	for(i = 0;i < m_server_list.Length();i++){
		m_server.AddString(m_server_list[i]);
	}
	
	// Make the first list item the currently selected line
	m_server.SetCurSel(0);

	// Initialize the PDB query list
	for(i = 0;i < m_query_list.Length();i++){
		m_pdb_id.AddString(m_query_list[i]);
	}
	
	// Make the first list item the currently selected line
	m_pdb_id.SetCurSel(0);

	// Set the uncompressed pdb query format
	//m_format.Format("http://%" SERVER_STR "%/pdb/cgi/export.cgi?format=PDB"
	//	"&compression=None&pdbId=%" QUERY_STR "%");

	// Set the compressed pdb query format
	m_format.Format("http://%" SERVER_STR "%/pdb/cgi/export.cgi?format=PDB"
		"&compression=gz&pdbId=%" QUERY_STR "%");

	UpdateData(FALSE);

	return TRUE;
}

void CNetworkPBDDialog::OnOK() 
{
	UpdateData(TRUE);

	CString tmp_server, tmp_query;
	CString tmp_request;

	m_server.GetWindowText(tmp_server);
	m_pdb_id.GetWindowText(tmp_query);

	if(!ValidatQueryFormat()){
		AfxMessageBox("Invalid query format!");
		return;
	}

	// Construct the request
	ConstructRequest(tmp_request, tmp_server, tmp_query);

	// Make a temporary file name to hold the downloaded file
	if(tmpnam(tmp_pdb_file) == NULL){
		AfxMessageBox("Can't make a temporary file to store pdb data!");
		return;
	}
	
	// Set the hourglass cursor - this may take a while
	::SetCursor(LoadCursor(NULL, IDC_WAIT));

	if(URLDownloadToFile(NULL, LPCSTR(tmp_request), tmp_pdb_file, 0, this) != S_OK){

		// Set the arrow cursor
		::SetCursor(LoadCursor(NULL, IDC_ARROW));

		AfxMessageBox("Unable to download pdb entry -- please check server and pdb id");
		return;
	}
	
	// Set the arrow cursor
	::SetCursor(LoadCursor(NULL, IDC_ARROW));

	// Save the query so we can display it in the title bar
	tmp_PDB_Id = tmp_query;

	// Update the server and pdb lists
	if(m_pdb_id.FindString(-1, tmp_query) == CB_ERR){
		m_query_list.Push(tmp_query);
	}

	if(m_server.FindString(-1, tmp_server) == CB_ERR){
		m_server_list.Push(tmp_server);
	}

	CDialog::OnOK();
}

BOOL CNetworkPBDDialog::ValidatQueryFormat()
{
	
	if(strstr(m_format, SERVER_STR) == NULL)
		return FALSE;

	if(strstr(m_format, QUERY_STR) == NULL)
		return FALSE;

	return TRUE;
}

void CNetworkPBDDialog::ConstructRequest(CString &tmp_request, CString &tmp_server, CString &tmp_query)
{
	tmp_request = m_format;

	tmp_request.Replace(SERVER_STR, LPCSTR(tmp_server));
	tmp_request.Replace(QUERY_STR, LPCSTR(tmp_query));

	//m_request.Format("http://www.rcsb.org/pdb/cgi/export.cgi?format=PDB&compression=None&pdbId=1ckw");
}
