//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/
// 
#include "stdafx.h"
#include "QmolDoc.h"
#include "OpenCoordTrajec.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// COpenCoordTrajec dialog


COpenCoordTrajec::COpenCoordTrajec(CWnd* pParent /*=NULL*/)
	: CDialog(COpenCoordTrajec::IDD, pParent)
{
	// The default filenames
	coord_filename = _T("");
	trajec_filename = _T("");

	// The default file types
	coord_filetype = QM_PDB;
	trajec_filetype = 0; // QM_DCD doesn't equal 0, the first item in the
						 // the combobox resource
	
	// By default, don't process the trajectory
	traj_process = QM_VERBATIM;

	//{{AFX_DATA_INIT(COpenCoordTrajec)
	//}}AFX_DATA_INIT
}

BOOL COpenCoordTrajec::OnInitDialog()
{
	CDialog::OnInitDialog();

	m_coord_type.SetCurSel(coord_filetype);
	m_trajec_type.SetCurSel(trajec_filetype);

	switch(traj_process){
		case QM_VERBATIM:
			CheckRadioButton(IDC_RADIO_VERBATIM,	// First button
							 IDC_RADIO_KABSCH,		// Last bottom	
							 IDC_RADIO_VERBATIM);	// Button to check
			break;
		case QM_CENTER_ALL:
			CheckRadioButton(IDC_RADIO_VERBATIM,	// First button
							 IDC_RADIO_KABSCH,		// Last bottom	
							 IDC_RADIO_CM);			// Button to check
			break;
		case QM_KABSCH:
			CheckRadioButton(IDC_RADIO_VERBATIM,	// First button
							 IDC_RADIO_KABSCH,		// Last bottom	
							 IDC_RADIO_KABSCH);		// Button to check
			break;
	};

	return TRUE;
}


void COpenCoordTrajec::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(COpenCoordTrajec)
	DDX_Control(pDX, IDC_OPEN_COORD_TRAJEC_NAME_T, m_trajec_name);
	DDX_Control(pDX, IDC_OPEN_COORD_TRAJEC_NAME_C, m_coord_name);
	DDX_Control(pDX, IDC_OPEN_COORD_TRAJEC_COMBO_T, m_trajec_type);
	DDX_Control(pDX, IDC_OPEN_COORD_TRAJEC_COMBO_C, m_coord_type);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(COpenCoordTrajec, CDialog)
	//{{AFX_MSG_MAP(COpenCoordTrajec)
	ON_BN_CLICKED(IDC_OPEN_COORD_TRAJEC_BROWSE_C, OnOpenCoordTrajecBrowseCoord)
	ON_BN_CLICKED(IDC_OPEN_COORD_TRAJEC_BROWSE_T, OnOpenCoordTrajecBrowseTrajec)
	ON_BN_CLICKED(IDC_RADIO_CM, OnRadioCm)
	ON_BN_CLICKED(IDC_RADIO_KABSCH, OnRadioKabsch)
	ON_BN_CLICKED(IDC_RADIO_VERBATIM, OnRadioVerbatim)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COpenCoordTrajec message handlers

void COpenCoordTrajec::OnOpenCoordTrajecBrowseCoord() 
{
	// Get the current file type (to set the filter in the
	// open file dialog).
	coord_filetype = m_coord_type.GetCurSel();

	// Open a file dialog here to load a file name
	CFileDialog fDlg(TRUE);
	
	switch(coord_filetype){
		case QM_PDB:
			fDlg.m_ofn.lpstrFilter = 
				"PDB files\0*.pdb\0All files\0*.*\0\0";
			break;
		case QM_XYZ:
			fDlg.m_ofn.lpstrFilter = 
				"XYZ files\0*.xyz\0All files\0*.*\0\0";
			break;
		case QM_CRD:
			fDlg.m_ofn.lpstrFilter = 
				"CRD files\0*.crd\0All files\0*.*\0\0";
			break;
	};
	
	// Set the current directory
	char dirBuffer[1024];

	// If the buffer is not big enought, DON'T set the current 
	// directory
	if(GetCurrentDirectory(1023, dirBuffer) != 0)
		fDlg.m_ofn.lpstrInitialDir = dirBuffer;

	if(fDlg.DoModal() == IDOK){
		// Load the selected file name into the filename
		// text box.
		coord_filename = fDlg.GetPathName();
		m_coord_name.SetWindowText(coord_filename);
	}
}

void COpenCoordTrajec::OnOpenCoordTrajecBrowseTrajec() 
{	
	// Get the current file type (to set the filter in the
	// open file dialog).
	trajec_filetype = m_trajec_type.GetCurSel() + QM_DCD;

	// Open a file dialog here to load a file name
	CFileDialog fDlg(TRUE);
	
	switch(trajec_filetype){
		case QM_DCD:
			fDlg.m_ofn.lpstrFilter = 
				"DCD files\0*.DCD\0All files\0*.*\0\0";
			break;
	};

	// Set the current directory
	char dirBuffer[1024];

	// If the buffer is not big enought, DON'T set the current 
	// directory
	if(GetCurrentDirectory(1023, dirBuffer) != 0)
		fDlg.m_ofn.lpstrInitialDir = dirBuffer;

	if(fDlg.DoModal() == IDOK){
		// Load the selected file name into the filename
		// text box.
		trajec_filename = fDlg.GetPathName();
		m_trajec_name.SetWindowText(trajec_filename);
	}
}

void COpenCoordTrajec::OnCancel() 
{	
	CDialog::OnCancel();
}

void COpenCoordTrajec::OnOK() 
{
	// Save the file name and type
	m_coord_name.GetWindowText(coord_filename);
	m_trajec_name.GetWindowText(trajec_filename);
	coord_filetype = m_coord_type.GetCurSel();
	trajec_filetype = m_trajec_type.GetCurSel();

	CDialog::OnOK();
}

CString COpenCoordTrajec::GetCoordFileName()
{
	return coord_filename;
}

CString COpenCoordTrajec::GetTrajecFileName()
{
	return trajec_filename;
}

int COpenCoordTrajec::GetCoordFileType()
{
	return coord_filetype;
}

int COpenCoordTrajec::GetTrajecFileType()
{
	return trajec_filetype + QM_DCD;
}

int COpenCoordTrajec::GetTrajecProcess()
{
	return traj_process;
}

void COpenCoordTrajec::OnRadioCm() 
{
	traj_process = QM_CENTER_ALL;
}

void COpenCoordTrajec::OnRadioKabsch() 
{
	traj_process = QM_KABSCH;
}

void COpenCoordTrajec::OnRadioVerbatim() 
{
	traj_process = QM_CENTER_ALL;
}
