//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/

#include "stdafx.h"
#include "Qmol.h"
#include "PasteDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPasteDialog dialog


CPasteDialog::CPasteDialog(int num_input, int num_atoms, int num_ca, 
						   CWnd* pParent /*=NULL*/)
	: CDialog(CPasteDialog::IDD, pParent)
{
	// Save the various atom and input counts for later error testing
	ca = num_ca;
	inputs = num_input;
	atoms = num_atoms;

	// By default, apply data to all atoms
	paste_to = PASTE_TO_ALL;

	//{{AFX_DATA_INIT(CPasteDialog)
	m_single_atom_id = 1;
	m_num_atoms = _T("");
	m_num_alpha = _T("");
	m_num_input = _T("");
	m_paste_as = PASTE_AS_T;	// By default, paste data to temperature factors
	m_atom_range_high = atoms;
	m_atom_range_low = 1;
	//}}AFX_DATA_INIT
}


void CPasteDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPasteDialog)
	DDX_Text(pDX, IDC_EDIT_ONE, m_single_atom_id);
	DDX_Text(pDX, IDC_STATIC_NUM_ATOMS, m_num_atoms);
	DDX_Text(pDX, IDC_STATIC_NUM_C_A, m_num_alpha);
	DDX_Text(pDX, IDC_STATIC_NUM_INPUT, m_num_input);
	DDX_CBIndex(pDX, IDC_COMBO_PASTE, m_paste_as);
	DDX_Text(pDX, IDC_EDIT_RANGE_HIGH, m_atom_range_high);
	DDX_Text(pDX, IDC_EDIT_RANGE_LOW, m_atom_range_low);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CPasteDialog, CDialog)
	//{{AFX_MSG_MAP(CPasteDialog)
	ON_BN_CLICKED(IDC_RADIO_PASTE_ALL, OnRadioPasteAll)
	ON_BN_CLICKED(IDC_RADIO_PASTE_ALL_C_A, OnRadioPasteAllCA)
	ON_BN_CLICKED(IDC_RADIO_PASTE_ONE, OnRadioPasteOne)
	ON_BN_CLICKED(IDC_RADIO_PASTE_RANGE, OnRadioPasteRange)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPasteDialog message handlers
BOOL CPasteDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Initialize the static strings
	m_num_atoms.Format("Number of atoms: %d", atoms);
	m_num_alpha.Format("Number of alpha carbons: %d", ca);
	m_num_input.Format("Number of inputs on clipboad: %d", inputs);
	
	// Select the correct radio button
	switch(paste_to){
		case PASTE_TO_ALL:
			CheckRadioButton(IDC_RADIO_PASTE_ALL,	// First button
							 IDC_RADIO_PASTE_RANGE,	// Last bottom	
							 IDC_RADIO_PASTE_ALL);
			break;
		case PASTE_TO_SINGLE:
			CheckRadioButton(IDC_RADIO_PASTE_ALL,	// First button
							 IDC_RADIO_PASTE_RANGE,	// Last bottom	
							 IDC_RADIO_PASTE_ONE);
			break;
		case PASTE_TO_RANGE:
			CheckRadioButton(IDC_RADIO_PASTE_ALL,	// First button
							 IDC_RADIO_PASTE_RANGE,	// Last bottom	
							 IDC_RADIO_PASTE_RANGE);
			break;
		case PASTE_TO_CA:
			CheckRadioButton(IDC_RADIO_PASTE_ALL,	// First button
							 IDC_RADIO_PASTE_RANGE,	// Last bottom	
							 IDC_RADIO_PASTE_ALL_C_A);
			break;
	};

	Activate();

	UpdateData(FALSE);

	return TRUE;
}

void CPasteDialog::Activate()
{
	switch(paste_to){
		case PASTE_TO_ALL:
			GetDlgItem(IDC_EDIT_ONE)->EnableWindow(FALSE);
			GetDlgItem(IDC_EDIT_RANGE_HIGH)->EnableWindow(FALSE);
			GetDlgItem(IDC_EDIT_RANGE_LOW)->EnableWindow(FALSE);
			break;
		case PASTE_TO_SINGLE:
			GetDlgItem(IDC_EDIT_ONE)->EnableWindow(TRUE);
			GetDlgItem(IDC_EDIT_RANGE_HIGH)->EnableWindow(FALSE);
			GetDlgItem(IDC_EDIT_RANGE_LOW)->EnableWindow(FALSE);
			break;
		case PASTE_TO_RANGE:
			GetDlgItem(IDC_EDIT_ONE)->EnableWindow(FALSE);
			GetDlgItem(IDC_EDIT_RANGE_HIGH)->EnableWindow(TRUE);
			GetDlgItem(IDC_EDIT_RANGE_LOW)->EnableWindow(TRUE);
			break;
		case PASTE_TO_CA:
			GetDlgItem(IDC_EDIT_ONE)->EnableWindow(FALSE);
			GetDlgItem(IDC_EDIT_RANGE_HIGH)->EnableWindow(FALSE);
			GetDlgItem(IDC_EDIT_RANGE_LOW)->EnableWindow(FALSE);
			break;
	};
}

void CPasteDialog::OnOK() 
{
	UpdateData(TRUE);
	
	// Validate the user inputs
	switch(paste_to){
		case PASTE_TO_ALL:
			if(inputs < atoms){
				AfxMessageBox("Unable to paste data, number of inputs is less\n"
					"than the number of atoms");
				return;
			}
			break;
		case PASTE_TO_SINGLE:
			// Make sure that the user has entered a number!
			break;
		case PASTE_TO_RANGE:
			// Make sure that the user has entered a pair of numbers!
			break;
		case PASTE_TO_CA:
			if(inputs < ca){
				AfxMessageBox("Unable to paste data, number of inputs is less\n"
					"than the number of alpha carbons");
				return;
			}
			break;
	};

	CDialog::OnOK();
}

void CPasteDialog::OnRadioPasteAll() 
{
	if(paste_to != PASTE_TO_ALL){
		paste_to = PASTE_TO_ALL;
		Activate();
	}
}

void CPasteDialog::OnRadioPasteAllCA() 
{
	if(paste_to != PASTE_TO_CA){
		paste_to = PASTE_TO_CA;
		Activate();
	}
}

void CPasteDialog::OnRadioPasteOne() 
{
	if(paste_to != PASTE_TO_SINGLE){
		paste_to = PASTE_TO_SINGLE;
		Activate();
	}
}

void CPasteDialog::OnRadioPasteRange() 
{
	if(paste_to != PASTE_TO_RANGE){
		paste_to = PASTE_TO_RANGE;
		Activate();
	}
}
