//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/
// 
#include "stdafx.h"
#include "Qmol.h"
#include "PrintOption.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPrintOption dialog


CPrintOption::CPrintOption(unsigned mem_size, int status, int clr_depth)
	: CDialog(CPrintOption::IDD, NULL)
{
	mem_status = status;
	depth = clr_depth;

	//{{AFX_DATA_INIT(CPrintOption)
	m_mem_size = mem_size;
	m_clr_depth = -1;
	//}}AFX_DATA_INIT
}


void CPrintOption::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPrintOption)
	DDX_Text(pDX, IDC_MEM_SIZE, m_mem_size);
	DDX_CBIndex(pDX, IDC_CLR_DEPTH_COMBO, m_clr_depth);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CPrintOption, CDialog)
	//{{AFX_MSG_MAP(CPrintOption)
	ON_BN_CLICKED(IDC_RADIO_LIMIT, OnRadioLimit)
	ON_BN_CLICKED(IDC_RADIO_NO_LIMIT, OnRadioNoLimit)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPrintOption message handlers

BOOL CPrintOption::OnInitDialog()
{
	CDialog::OnInitDialog();

	if(mem_status == MEM_LIMIT)
		CheckRadioButton(IDC_RADIO_NO_LIMIT,	// First button
						 IDC_RADIO_LIMIT,		// Last bottom	
						 IDC_RADIO_LIMIT);		// Button to check
	else
		CheckRadioButton(IDC_RADIO_NO_LIMIT,	// First button
						 IDC_RADIO_LIMIT,		// Last bottom	
						 IDC_RADIO_NO_LIMIT);	// Button to check

	switch(depth){
		case 16:
			m_clr_depth = 0;
			break;
		case 24:
			m_clr_depth = 1;
			break;
		default: // 16 bit if in doubt!
			m_clr_depth = 0;
			break;
	};

	UpdateData(FALSE);

	return TRUE;
}

void CPrintOption::OnOK() 
{
	UpdateData(TRUE);

	switch(m_clr_depth){
		case 0:
			depth = 16;
			break;
		case 1:
			depth = 24;
			break;
		default: // 16 bit if in doubt!
			depth = 0;
			break;
	};

	CDialog::OnOK();
}

void CPrintOption::OnRadioLimit() 
{
	mem_status = MEM_LIMIT;
}

void CPrintOption::OnRadioNoLimit() 
{
	mem_status = MEM_NO_LIMIT;
}
