//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/

#ifndef __QMOL_ATOM
#define __QMOL_ATOM

#include "QmolDefine.h"

class atom{
	public:
		// Default constructor
		atom()
		{
			name = NULL;
			elem = ELEM_DUMMY;
			id = -1;
			radius = 1.0;
			mass = 1.0;
			coord[0] = coord[1] = coord[2] = 0.0;
			resName = NULL;
			resSeq = 0;
			temperature = 0.0;
			occupancy = 0.0;
			rgb_color[0] = rgb_color[1] = rgb_color[2] = 
				rgb_color[3] = 255; // Opaque white
			isSolvent = FALSE;
			model = 1;
			chain = 1;
		};
		
		~atom()
		{
			if(name){
				delete [] name;
				name = NULL;
			}
			
			if(resName){
				delete [] resName;
				resName = NULL;
			}
		};
		
		atom& operator=(const atom &a)
		{
			radius = a.radius;
			mass = a.mass;
			id = a.id;

			coord[0] = a.coord[0];
			coord[1] = a.coord[1];
			coord[2] = a.coord[2];

			resSeq = a.resSeq;
			temperature = a.temperature;
			occupancy = a.occupancy;
			elem = a.elem;
			rgb_color[0] = a.rgb_color[0];
			rgb_color[1] = a.rgb_color[1];
			rgb_color[2] = a.rgb_color[2];
			rgb_color[3] = a.rgb_color[3];
			isSolvent = a.isSolvent;
			model = a.model;
			chain = a.chain;

			if(a.name){
				name = new char [strlen(a.name) + 1];
				strcpy(name, a.name);
			}
			else
				name = NULL;
			
			if(a.resName){
				resName = new char [strlen(a.resName) + 1];
				strcpy(resName, a.resName);
			}
			else
				resName = NULL;
				
			return (*this);
		};
		
		// Convert an element name to an element index
		void GetElement(char *elem_name);
		
		char *name;
		float radius, mass;
		//float x, y, z;
		float coord[3];
		
		// Fields borrowed from the pdb format
		char *resName;
		int resSeq;
		float temperature;
		float occupancy;
		
		// The atomic id number (i.e. serial number in a
		// pdb file).
		int id;
		
		// The element type
		unsigned int elem;
		
		// The atomic color
		unsigned char rgb_color[4];
		
		// Is this atom part of a solvent molecule?
		BOOL isSolvent;

		// The model number (for multiple NMR-style structures).
		int model;

		// The chain ID for multiple chains
		char chain;

		// Set the atom color by element, according to the given coloring scheme
		void SetAtomColor(unsigned char color_scheme[ELEM_LAST][4]);
		void SetAtomColor(float clr_index);

		void SetAtomRadius(float radius_scheme[]);
		void SetAtomMass(float mass_scheme[]);

		inline void GetCoord(float r[])
		{
			memcpy(r, coord, 3*sizeof(float));
		};

		// Return the (distance/radius)^2 from the point {rx, ry, rz} to the
		// center of the atom. This quantity is used when computing
		// molecular surfaces.
		inline float GetDist2(float rx, float ry, float rz)
		{
			rx -= coord[0];
			ry -= coord[1];
			rz -= coord[2];

			return (rx*rx + ry*ry + rz*rz);
		};
};

#endif // __QMOL_ATOM
