//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/
// 
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_QMOLVIEW_H__979F19CE_DCC9_11D3_9099_00A0C9A3671F__INCLUDED_)
#define AFX_QMOLVIEW_H__979F19CE_DCC9_11D3_9099_00A0C9A3671F__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "Qmol.h"
#include "QmolDoc.h"
#include "AdjustTorsionDialog.h"
#include "MeasureDialog.h"
#include "AnimateDialog.h"
#include "QMColorDialog.h"
#include "trackball.h"
#include "FlexColorDialog.h"
#include "AtomRadiusDialog.h"
#include "QMLight.h"
#include "LightDialog.h"
#include "CustomResDialog.h"
#include "LinearAlgebra3D.h"
#include "ClipPlane.h"
#include "SequenceDialog.h"
#include "SurfDialog.h"
#include "MoleculeSelectDialog.h"
#include "SelectObject.h"
#include "AtomSelectionDialog.h"

#ifdef _DEBUG
extern BOOL debug_flag;
#endif //_DEBUG

// Define COMPUTE_FPS to compute the number of frames per sec
//#define	COMPUTE_FPS

class moleculeSpline{
	private:
		// A pointer to a molecule
		molecule *mol;
		int cur_knot;
		int num_knots;
		
		BOOL update;

		GLfloat t, dt;
		
		// A spline control parameter
		GLfloat gamma, inv_gamma;
	
		// Which chain/model are we interapolating?
		int chain;

		GLfloat TM[4];

		// Atom and guide positions
		GLfloat G_c[4][3];
		GLfloat G_g[4][3];

		// Ribbon dimensions
		GLfloat G_major[4];
		GLfloat G_minor[4];

		// The derivative of the interpolated trace with
		// respect to the interpolation parameter.
		GLfloat dr[3];

		// Temporary vectors used for storing intermediate
		// calculations (cross products etc.).
		GLfloat A[3];
		GLfloat B[3];

		// The current orientation for this segment of the spline
		GLfloat guide_ref[3];

		GLfloat tmp[3];
	public:
		moleculeSpline(molecule *m_mol, int m_chain, int m_res, GLfloat m_gamma = 2.0f);

		void Reset();

		inline BOOL NextTrace(GLfloat r[], int &index, int cur_step);
		inline BOOL NextTrace(GLfloat r[], int &index);

		inline BOOL NextRibbon(GLfloat r[], GLfloat major[], 
			GLfloat minor[], int &index);

		inline BOOL NextRibbon(GLfloat r[], GLfloat major[], 
			GLfloat minor[], int &index, int cur_step);

		inline int CurKnot()
		{
			return cur_knot;
		};

		inline void GetNormal(GLfloat m_dr[], BOOL head)
		{
			if(head){
				// If we're at the head of the interpolation, flip
				// the normal
				m_dr[X] = -dr[X];
				m_dr[Y] = -dr[Y];
				m_dr[Z] = -dr[Z];
			}
			else{
				m_dr[X] = dr[X];
				m_dr[Y] = dr[Y];
				m_dr[Z] = dr[Z];
			}
		};

		// Are we at least half way to the next knot?
		inline BOOL isHalfWay()
		{
			if(t - dt > 0.5f)
				return TRUE;

			return FALSE;
		};
};


class CQmolView : public CView
{
protected: // create from serialization only
	CQmolView();
	DECLARE_DYNCREATE(CQmolView)
	
	void Update();
// Attributes
public:
	CQmolDoc* GetDocument();

// Operations
public:
// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CQmolView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual void OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint);
	protected:
	virtual void OnPrint(CDC* pDC, CPrintInfo* pInfo);
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	//}}AFX_VIRTUAL

public:
	//////////////////////////////////
	// External OpenGL state variables
	//////////////////////////////////

	// Number of bits color depth for printing
	int print_color_depth;

	float mat_specular[4];
	float mat_diffuse[4];
	float mat_shininess[1];

	// Model a uniform (ambient) light source + specular highlights for the
	// selection spheres

	// Lighting
	LightSource ls_wire_frame;
	LightBox lb_solid;

	int color_scheme;	

	// Associate a flag with the different colors
	enum {BACKGROUND_CLR, SELECTION_CLR, HBOND_CLR, CLIP_PLANE_CLR};

	// Background color
	float background[4];

	// The selection color (for torsion angles and
	// the lines connecting selected atoms).
	float selection_color[4];
	
	// The hydrogen bond color
	float hbond_color[3];

	// Should we use depth cuing?
	int depthCue;
	
	// Should we use anti-aliased lines?
	int lineAlias;

	// What resolution should we use for rendering 3D objects
	// (low, medium or high for now).
	int atomRes, bondRes, selectionRes, splineRes, ribbonRes;
	
	float splineTension;
	
	float line_width;

	int resolution;

	// The ball-and-stick sizes
	float stickRadius;
	float ballRadius;

	// Enable smooth verticies for 3D objects?
	int smooth3D;

	// Fog for depth cueing
	float fogDensity;
	
	// OpenGL perspective for zooming
	float perspective;
	
	// The size of the largest dimension
	float maxDim;

	// The coordinates that will be mapped to the 
	// center of the view
	float center[3];

	// Molecule rotation and scaling
	int beginScaleX;

	// Should we display axes?
	int axes;

	// Should the "box" that contains the molecules be scaled
	// when molecule are moved (i.e. when their center of mass is
	// no longer at the origin)?
	BOOL scaleToFit;

	// Should the molecules sping freely?
	int momentum;
	int spinning;

	// The orientation of the view reference frame
	DDD referenceFrame;
	
	// Allow the user to interactively adjust torsion angles
	AdjustTorsion tor_adjust;

	// Is the left button down?
	int l_button_down;

	// Is the shift key down?
	int shift_key_down;
	
	// Window dimensions
	int viewWidth;
	int viewHeight;

	// Display a stero view of the current molecule?
	BOOL displayStereo;
	BOOL renderSplitStereoRightEye;
	BOOL renderSplitStereoLeftEye;
	float stereoOffset;

	// Bits-per-pixel for the OpenGL subsystem
	int bpp;

	// Number of bits to use for the OpenGL Z buffer
	int zbuf;

	int animateTraj;
	int animateDirection;

	// Display hydrogen bonds?
	int showHBonds;


	// Clipping information
	ClipPlane clipper;	// The clipping engine
	BOOL clipView;		// --> Clip everything
	BOOL clipSurface;	// --> Only clip the surface
	BOOL displayClipPlane;
	GLfloat clipPlaneClr[4];

#ifdef COMPUTE_FPS
	// Compute FPS
	int fps_fames, fps_cur_frame;
	unsigned long fps_time;
	float fps;

	void renderFPS();
#endif // COMPUTE_FPS

// Implementation
public:
	// Update the status bar
	void UpdateStatusBar(SelectObj &atom);

	// Save a meta file to the windows clip board
	void SaveMetaFile();

	// Create a device independant bitmap
	HBITMAP CreateDIB(CDC* pDC);

	// Restore application defaults
	void LoadDefaults(BOOL m_useRegistry);
	void LoadRegistry();
	void SaveRegistry();
	void CleanRegistry();

	virtual ~CQmolView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
	void GetOpenGLError(char *m_file, int m_line);
#endif // _DEBUG

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CQmolView)
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnDestroy();
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnMButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnAdjustTorsionangle();
	afx_msg LRESULT OnAdjustTorsionangleClose(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnAdjustTorsionangleModify(WPARAM wParam, LPARAM lParam);
	afx_msg void OnOptionsMeasure();
	afx_msg LRESULT OnOptionsMeasureClose(WPARAM wParam, LPARAM lParam);
	afx_msg void OnOptionsCenter();
	afx_msg void OnRenderAtomLabels();
	afx_msg void OnUpdateRenderAtomLabels(CCmdUI* pCmdUI);
	afx_msg void OnOptionsFont();
	afx_msg void OnOptionsColor();
	afx_msg void OnRenderSolvent();
	afx_msg void OnUpdateRenderSolvent(CCmdUI* pCmdUI);
	afx_msg void OnOptionsMomentum();
	afx_msg void OnUpdateOptionsMomentum(CCmdUI* pCmdUI);
	afx_msg void OnOptionsAnimate();
	afx_msg LRESULT OnAnimateClose(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnAnimateAdjust(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnAnimateForward(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnAnimateStop(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnAnimateReverse(WPARAM wParam, LPARAM lParam);
	afx_msg void OnRenderDepthCue();
	afx_msg void OnUpdateRenderDepthCue(CCmdUI* pCmdUI);
	afx_msg void OnRenderCpk();
	afx_msg void OnUpdateRenderCpk(CCmdUI* pCmdUI);
	afx_msg void OnRenderWireFrame();
	afx_msg void OnUpdateRenderWireFrame(CCmdUI* pCmdUI);
	afx_msg void OnRenderAliasLine();
	afx_msg void OnUpdateRenderAliasLine(CCmdUI* pCmdUI);
	afx_msg void OnRenderHighRes();
	afx_msg void OnUpdateRenderHighRes(CCmdUI* pCmdUI);
	afx_msg void OnRenderLowRes();
	afx_msg void OnUpdateRenderLowRes(CCmdUI* pCmdUI);
	afx_msg void OnRenderMedRes();
	afx_msg void OnUpdateRenderMedRes(CCmdUI* pCmdUI);
	afx_msg void OnRenderSmooth3d();
	afx_msg void OnUpdateRenderSmooth3d(CCmdUI* pCmdUI);
	afx_msg void OnRenderPoint();
	afx_msg void OnUpdateRenderPoint(CCmdUI* pCmdUI);
	afx_msg void OnFilePrintOption();
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnMButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnRButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnRenderMonitor();
	afx_msg void OnUpdateRenderMonitor(CCmdUI* pCmdUI);
	afx_msg void OnRenderStick();
	afx_msg void OnUpdateRenderStick(CCmdUI* pCmdUI);
	afx_msg void OnRenderBallAndStick();
	afx_msg void OnUpdateRenderBallAndStick(CCmdUI* pCmdUI);
	afx_msg void OnEditCopy();
	afx_msg void OnOptionsAtomColor();
	afx_msg LRESULT OnOptionsColorClose(WPARAM wParam, LPARAM lParam);
	afx_msg void OnRenderAxes();
	afx_msg void OnUpdateRenderAxes(CCmdUI* pCmdUI);
	afx_msg LRESULT OnFlexColorMsg(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnSequenceDialogMsg(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnAtomSelectDialogMsg(WPARAM wParam, LPARAM lParam);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnAdjustRadius();
	afx_msg LRESULT OnAdjustRadiusClose(WPARAM wParam, LPARAM lParam);
	afx_msg void OnOptionsSelectColor();
	afx_msg void OnRenderLighting();
	afx_msg LRESULT OnLightingMessage(WPARAM wParam, LPARAM lParam);
	afx_msg void OnFileMovie();
	afx_msg void OnUpdateFileMovie(CCmdUI* pCmdUI);
	afx_msg void OnFileCoordOpen();
	afx_msg void OnFileCoordSave();
	afx_msg void OnFileOpenCoordTraj();
	afx_msg LRESULT OnResolutionMessage(WPARAM wParam, LPARAM lParam);
	afx_msg void OnRenderCustomRes();
	afx_msg void OnUpdateRenderCustomRes(CCmdUI* pCmdUI);
	afx_msg void OnRenderTraceStick();
	afx_msg void OnUpdateRenderTraceStick(CCmdUI* pCmdUI);
	afx_msg void OnRenderTraceWireframe();
	afx_msg void OnUpdateRenderTraceWireframe(CCmdUI* pCmdUI);
	afx_msg void OnRenderTraceSpline();
	afx_msg void OnUpdateRenderTraceSpline(CCmdUI* pCmdUI);
	afx_msg void OnRenderLineWidth1();
	afx_msg void OnUpdateRenderLineWidth1(CCmdUI* pCmdUI);
	afx_msg void OnRenderLineWidth2();
	afx_msg void OnUpdateRenderLineWidth2(CCmdUI* pCmdUI);
	afx_msg void OnRenderLineWidth3();
	afx_msg void OnUpdateRenderLineWidth3(CCmdUI* pCmdUI);
	afx_msg void OnRenderLineWidth4();
	afx_msg void OnUpdateRenderLineWidth4(CCmdUI* pCmdUI);
	afx_msg void OnRenderLineWidth5();
	afx_msg void OnUpdateRenderLineWidth5(CCmdUI* pCmdUI);
	afx_msg void OnRenderRibbon();
	afx_msg void OnUpdateRenderRibbon(CCmdUI* pCmdUI);
	afx_msg void OnRenderRibbonFlat();
	afx_msg void OnUpdateRenderRibbonFlat(CCmdUI* pCmdUI);
	afx_msg void OnRenderTube();
	afx_msg void OnUpdateRenderTube(CCmdUI* pCmdUI);
	afx_msg void OnRenderTubeOcc();
	afx_msg void OnUpdateRenderTubeOcc(CCmdUI* pCmdUI);
	afx_msg void OnRenderTubeT();
	afx_msg void OnUpdateRenderTubeT(CCmdUI* pCmdUI);
	afx_msg void OnRenderRibbonShowWire();
	afx_msg void OnUpdateRenderRibbonShowWire(CCmdUI* pCmdUI);
	afx_msg void OnOptionsStereo();
	afx_msg void OnUpdateOptionsStereo(CCmdUI* pCmdUI);
	afx_msg void OnNetCoordOpen();
	afx_msg void OnFileTrajExport();
	afx_msg void OnUpdateFileCoordSave(CCmdUI* pCmdUI);
	afx_msg void OnUpdateFileTrajExport(CCmdUI* pCmdUI);
	afx_msg void OnRenderHbonds();
	afx_msg void OnUpdateRenderHbonds(CCmdUI* pCmdUI);
	afx_msg void OnOptionsHbondColor();
	afx_msg BOOL OnMouseWheel(UINT nFlags, short zDelta, CPoint pt);
	afx_msg void OnOptionsMouseDec();
	afx_msg void OnOptionsMouseInc();
	afx_msg void OnOptionsMouseDef();
	afx_msg void OnEditPaste();
	afx_msg void OnUpdateEditPaste(CCmdUI* pCmdUI);
	afx_msg void OnRenderClipColor();
	afx_msg void OnRenderClipShow();
	afx_msg void OnRenderClipView();
	afx_msg void OnUpdateRenderClipShow(CCmdUI* pCmdUI);
	afx_msg void OnUpdateRenderClipView(CCmdUI* pCmdUI);
	afx_msg void OnRenderClipCenter();
	afx_msg void OnOptionsStrucAssign();
	afx_msg void OnOptionsSequenceColor();
	afx_msg void OnExportImage();
	afx_msg void OnRenderSurface();
	afx_msg void OnUpdateRenderSurface(CCmdUI* pCmdUI);
	afx_msg void OnRenderClipSurface();
	afx_msg void OnUpdateRenderClipSurface(CCmdUI* pCmdUI);
	afx_msg void OnRenderSurfaceOpt();
	afx_msg LRESULT OnSurfaceOptMessage(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnSurfaceBuildMessage(WPARAM wParam, LPARAM lParam);
	afx_msg void OnOptionsSurfaceColor();
	afx_msg void OnRenderMesh();
	afx_msg void OnUpdateRenderMesh(CCmdUI* pCmdUI);
	afx_msg void OnRenderNone();
	afx_msg void OnUpdateRenderNone(CCmdUI* pCmdUI);
	afx_msg void OnRenderSurfacePoint();
	afx_msg void OnUpdateRenderSurfacePoint(CCmdUI* pCmdUI);
	afx_msg void OnMeasureSurfaceResidue();
	afx_msg void OnUpdateMeasureSurfaceResidue(CCmdUI* pCmdUI);
	afx_msg void OnMeasureSurfaceAtom();
	afx_msg void OnUpdateMeasureSurfaceAtom(CCmdUI* pCmdUI);
	afx_msg void OnOptionsDefault();
	afx_msg void OnOptionsPurgeReg();
	afx_msg void OnUpdateOptionsPurgeReg(CCmdUI* pCmdUI);
	afx_msg void OnRenderMonitorLabels();
	afx_msg void OnUpdateRenderMonitorLabels(CCmdUI* pCmdUI);
	afx_msg void OnOptionsMolSel();
	afx_msg void OnUpdateRenderSurfaceOpt(CCmdUI* pCmdUI);
	afx_msg void OnUpdateOptionsSurfaceColor(CCmdUI* pCmdUI);
	afx_msg void OnUpdateOptionsSequenceColor(CCmdUI* pCmdUI);
	afx_msg void OnUpdateOptionsAtomColor(CCmdUI* pCmdUI);
	afx_msg LRESULT OnMolSelectDialogMsg(WPARAM wParam, LPARAM lParam);
	afx_msg void OnOptionsScaleFit();
	afx_msg void OnUpdateOptionsScaleFit(CCmdUI* pCmdUI);
	afx_msg void OnUpdateOptionsStrucAssign(CCmdUI* pCmdUI);
	afx_msg void OnMoleculeAlign();
	afx_msg void OnUpdateMoleculeAlign(CCmdUI* pCmdUI);
	afx_msg void OnUpdateOptionsMeasure(CCmdUI* pCmdUI);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

public:
	// The following member funcntions have been added
	// to the App Wizard Framework.
	BOOL SetupPixelFormat(int bpp, int zbuf);
	BOOL SetupGLView(int m_width, int m_height);

	BOOL RenderScene(list<molecule *> &m_molList, int mode);

	GLuint GenerateBitmapListForFont(LOGFONT &font);
	GLuint GetNewDisplayListIDs(GLsizei range);
	BOOL SetMemDcPixelFormat(HDC hMemDC, int bpp);

	// Set the background color according to the current color scheme
	void SetBackgroundColor();
	void SetMeasureStipple();
	void SetHBondStipple();

	// Handle the nitty-gritty aspects of the rendering
	void renderAtoms(molecule *mol, GLfloat m[][4]);
	void renderWireFrame(molecule *mol);
	void renderBallAndStick(molecule *mol, GLfloat m[][4]);
	void renderVanDerWaals(molecule *mol, GLfloat m[][4]);
	void renderTraceWireFrame(molecule *mol);
	void renderTraceSpline(molecule *mol);
	void renderTraceStick(molecule *mol, GLfloat m[][4]);
	void renderRibbon(molecule *mol);
	void renderFlatRibbon(molecule *mole);
	void renderPoints(molecule *mol, GLfloat m[][4]);
	void RenderSurface(molecule *mol);

	void renderSelect(molecule *mol, int &offset);
	void renderMeasure();
	void renderTorsion();
	void renderMolecule(molecule *mol, int mode, int &offset);

	void renderLabels(molecule *mol);
	void renderMonitors(molecule *mol, GLfloat m[][4]);
	void renderHBonds();
	void renderAxes(float scale_x, float scale_y, float scale_z, GLfloat m[][4]);

	// Export the current molecular view in a particular image format
	void ExportImagePS(const char *filename);
	void ExportImageBMP(const char *filename);
	void ExportImagePNG(const char *filename);
	HBITMAP MakeDIB(CDC* pDC, int dib_width, int dib_height, int dib_bpp);

	// Request a window redraw from the OS
	void postRedisplay();
	
	// Molecule rotation and scaling
	void rotation(int x, int y);
	void startRotation(int x, int y);
	void stopSpinning();
	void StartScale(int x);
	void Scale(int x);
	void ScaleWheel(int num_rot);
	void StartMove(int x, int y);
	void Move(int x, int y);
	
	// Control the active/deactive flag of the adjust torsion
	// angle slider.
	void ActiveAdjustTorsionScale();

	// Set the initial OpenGL state
	BOOL InitializeOpenGL();

	// Build the geometric primitives based on the resolution
	// defined by m_res.
	void SetGeomObjRes(int m_res);

	// Build (or rebuild) geometric primitives based on the
	// current resolution.
	void BuildGeomObj();

	// Handle the OpenGL selection mechanism
	SelectObj pickScene(list<molecule *> &molList, int x, int y);
	SelectObj processHits(list<molecule *> &molList, GLint hits, 
		int bufSize, GLuint *buffer);

	inline CDC* Get_pDC()
	{
		return m_pDC;
	}

	inline molecule* GetActiveMol()
	{
		CQmolDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);

		return pDoc->activeMol;
	}

	// Provide a mechanism to halt the WM_PAINT message pump!
	inline void PauseAnimation(BOOL m_status)
	{
		static int paused = FALSE;
		static BOOL tmpAnimate = FALSE;
		static BOOL tmpAnimateTraj = FALSE;
		static BOOL tmpSpinning = FALSE;
		static int tmpCurStep = 0;

		// Don't allow the "unpause" unless 
		// previously paused!
		if(!m_status && !paused)
			return;

		molecule* ptrMol = GetActiveMol();
		
		if(!ptrMol)
			return;

		CQmolDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);

		if(m_status){
			tmpAnimate = animate;
			tmpAnimateTraj = animateTraj;
			tmpSpinning = spinning;
			tmpCurStep = pDoc->CurrentTrajStep();

			animate = FALSE;
			animateTraj = FALSE;
			spinning = FALSE;
			pDoc->CurrentTrajStep(0);

			paused = TRUE;
		}
		else{
			animate = tmpAnimate;
			animateTraj = tmpAnimateTraj;
			spinning = tmpSpinning;
			ptrMol->current_step = tmpCurStep;

			paused = FALSE;

			postRedisplay();
		}
	};

	// Does the clipboard contain data in format that Qmol
	// can import?
	UINT GetClipFormat();
	
	BOOL ParseClipData(HANDLE clipData, list<float> &clip_data);

private:
	CAdjustTorsionDialog *m_pATDlg;
	CMeasureDialog *m_pMDlg;
	CAnimateDialog *m_pAnimateDlg;
	CQMColorDialog *m_qmColorDlg;
	CFlexColorDialog *m_flexClrDlg;
	CAtomRadiusDialog *m_pARDlg;
	CLightDialog *m_lightDlg;
	CCustomResDialog *m_customResDlg;
	CSequenceDialog *m_seqDlg;
	CSurfDialog *m_surfDlg;
	CMoleculeSelectDialog *m_molSelDlg;
	CAtomSelectionDialog *m_atomSelectDlg;

	// The ID of the OpenGL call list for displaying fonts
	GLint font_list_id;

	// A Windows font structure for storing the current font.
	LOGFONT font;

	// Font scaling info
	int printScaleMul, printScaleDiv;

	HGLRC	m_hRC;
	CDC*	m_pDC;
	
	// Are we animating?
	BOOL animate;

	// Store the time since the last animation frame
	DWORD m_ElapsedTime;
	DWORD m_previousElapsedTime;

	// Maximum size of bitmap size when printing
	int mem_size;

	BOOL purgeRegistryEntries;
};

#ifndef _DEBUG  // debug version in QmolView.cpp
inline CQmolDoc* CQmolView::GetDocument()
   { return (CQmolDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_QMOLVIEW_H__979F19CE_DCC9_11D3_9099_00A0C9A3671F__INCLUDED_)
