//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/
// 
#include "stdafx.h"
#include "Qmol.h"
#include "ReadProgressDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Local function wrapper
UINT ThreadWrapper(LPVOID pParam);

/////////////////////////////////////////////////////////////////////////////
// CReadProgressDialog dialog


CReadProgressDialog::CReadProgressDialog(int min, int max, molecule *mol,
										 CString &filename)
	: CDialog(CReadProgressDialog::IDD, NULL)
{
	m_min = min;
	m_max = max;
	m_mol = mol;
	m_filename = filename;
	status = TRUE;
}


void CReadProgressDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CReadProgressDialog)
	DDX_Control(pDX, IDC_PROGRESS, m_progress);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CReadProgressDialog, CDialog)
	//{{AFX_MSG_MAP(CReadProgressDialog)
	ON_BN_CLICKED(ID_STOP, OnStop)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CReadProgressDialog message handlers

BOOL CReadProgressDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the range of the progress control
	// (use the 32 bit SetRange because the default
	// is the short).
	m_progress.SetRange32(m_min, m_max);
	m_progress.SetStep(1);

	// Start the thread that will read the trajectory data
	AfxBeginThread(ThreadWrapper, this);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CReadProgressDialog::OnStop() 
{
	// Tell the Document call that we've been 'Canceled'
	status = FALSE;
	
	// Don't call CDialog::OnCancel()! By setting status
	// to false, we will cause the reading thread to exit and
	// thereby close the window.
}

UINT ThreadWrapper(LPVOID pParam)
{
	// if pParam == NULL we're dead meat
	
	CReadProgressDialog* ptr = (CReadProgressDialog*)pParam;

	// Keep reading until we're read all of the data or the user
	// has pressed the cancel button.
	for(int i = ptr->m_min;i < ptr->m_max && ptr->Status();i++){
		ptr->m_mol->ReadTrajStep(ptr->m_filename, QM_DCD);
		
		// Increment the progress bar
		ptr->m_progress.StepIt();
	}

	// Close the window when we are done
	ptr->PostMessage(WM_CLOSE);

	return 0;

}