//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/
// 
#include "stdafx.h"
#include "QmolDoc.h"
#include "SaveCoordDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSaveCoordDialog dialog


CSaveCoordDialog::CSaveCoordDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CSaveCoordDialog::IDD, pParent)
{
	// The default filename
	filename = _T("");

	// The default file type
	filetype = QM_PDB;
	
	// The default precision
	precision = 3;

	//{{AFX_DATA_INIT(CSaveCoordDialog)
	m_write_conect = TRUE;
	m_normalize_t = TRUE;
	//}}AFX_DATA_INIT
}


BOOL CSaveCoordDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	m_file_type.SetCurSel(filetype);
	
	// The first precision option is 3
	m_precision.SetCurSel(precision - 3);

	return TRUE;
}

void CSaveCoordDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSaveCoordDialog)
	DDX_Control(pDX, IDC_SAVE_FILE_NAME, m_file_name);
	DDX_Control(pDX, IDC_SAVE_COORD_PREC_COMBO, m_precision);
	DDX_Control(pDX, IDC_SAVE_COORD_COMBO, m_file_type);
	DDX_Check(pDX, IDC_CONECT_CHECK, m_write_conect);
	DDX_Check(pDX, IDC_NORMALIZE_T_CHECK, m_normalize_t);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSaveCoordDialog, CDialog)
	//{{AFX_MSG_MAP(CSaveCoordDialog)
	ON_BN_CLICKED(IDC_SAVE_COORD_BROWSE, OnSaveCoordBrowse)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSaveCoordDialog message handlers

void CSaveCoordDialog::OnCancel() 
{
	CDialog::OnCancel();
}

void CSaveCoordDialog::OnOK() 
{
	UpdateData(TRUE);

	// Save the file name and type
	m_file_name.GetWindowText(filename);
	filetype = m_file_type.GetCurSel();
	precision = m_precision.GetCurSel() + 3;

	CDialog::OnOK();
}

void CSaveCoordDialog::OnSaveCoordBrowse() 
{
	// Open a file dialog here to load a file name
	CFileDialog fDlg(TRUE);
	
	// Get the current file type (to set the filter in the
	// open file dialog).
	filetype = m_file_type.GetCurSel();
	
	switch(filetype){
		case QM_PDB:
			fDlg.m_ofn.lpstrFilter = 
				"PDB files\0*.pdb\0All files\0*.*\0\0";
			break;
		case QM_XYZ:
			fDlg.m_ofn.lpstrFilter = 
				"XYZ files\0*.xyz\0All files\0*.*\0\0";
			break;
		case QM_CRD:
			fDlg.m_ofn.lpstrFilter = 
				"CRD files\0*.crd\0All files\0*.*\0\0";
			break;
	};

	// Set the current directory
	char dirBuffer[1024];

	// If the buffer is not big enought, DON'T set the current 
	// directory
	if(GetCurrentDirectory(1023, dirBuffer) != 0)
		fDlg.m_ofn.lpstrInitialDir = dirBuffer;

	if(fDlg.DoModal() == IDOK){
		// Load the selected file name into the filename
		// text box.
		filename = fDlg.GetPathName();
		m_file_name.SetWindowText(filename);
	}
}

CString CSaveCoordDialog::GetFileName()
{
	return filename;
}

int CSaveCoordDialog::GetFileType()
{
	return filetype;
}

int CSaveCoordDialog::GetPrecision()
{
	return precision;
}
