//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/

#ifndef __QMOL_SELECT_OBJECT
#define	__QMOL_SELECT_OBJECT

// A forward definition
class molecule;

class SelectObj {
private:
	int objType;
	int objIndex;

	// In which molecule is this object located?
	molecule *molPtr;
public:

	// Enumerate the different kinds of objects
	enum {NO_OBJ, ATOM_OBJ, BOND_OBJ};

	SelectObj()
	{
		molPtr = NULL;
		objType = NO_OBJ;
		objIndex = -1;
	};

	SelectObj(molecule *m_molPtr, int m_objType)
	{
		molPtr = m_molPtr;
		objType = m_objType;
		objIndex = -1;
	};

	inline void SetIndex(int m_index)
	{
		objIndex = m_index;
	};

	inline void SetMol(molecule *m_molPtr)
	{
		molPtr = m_molPtr;
	};

	inline void SetType(int m_objType)
	{
		objType = m_objType;
	};

	inline int GetIndex()
	{
		return objIndex;
	};

	inline molecule* GetMol()
	{
		return molPtr;
	};

	inline BOOL isNone()
	{
		return (objType == NO_OBJ);
	};

	inline BOOL isAtom()
	{
		return (objType == ATOM_OBJ);
	};

	inline BOOL isBond()
	{
		return (objType == BOND_OBJ);
	};

	inline operator==(SelectObj &m_obj)
	{
		if(objType != m_obj.objType)
			return FALSE;

		if(objIndex != m_obj.objIndex)
			return FALSE;

		if(molPtr != m_obj.molPtr)
			return FALSE;

		return TRUE;
	};
};

#endif // __QMOL_SELECT_OBJECT