//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/

// Code for measuring bond lengths, bond angles and torsion angles.

// A win32/MSDEV thing ...
#include "stdafx.h"

#include "Measure.h"
#include "QmolDoc.h"
#include "LinearAlgebra3D.h"
#include <math.h>

// Local function definitions
float GetBond(float x_i, float y_i, float z_i,
	       float x_j, float y_j, float z_j);
float GetBondAngle(float x_i, float y_i, float z_i,
		    float x_j, float y_j, float z_j,
		    float x_k, float y_k, float z_k);
float GetTorsionAngle(float x_i, float y_i, float z_i,
		       float x_j, float y_j, float z_j,
		       float x_k, float y_k, float z_k,
		       float x_l, float y_l, float z_l);
//
// Implement the dot product as a macro
//
#define	dot(x1,x2,x3,y1,y2,y3)	( x1*y1 + x2*y2 + x3*y3 )
			 
measure::measure()
{
	Flush();
}

void measure::Flush()
{
	while(!m_list.IsEmpty())
		m_list.Pop();
}

void measure::Flush(list<molecule*> &molList)
{
	ListIterator<AtomInfo> nextInfo(m_list);
	AtomInfo *tmpInfo;
	
	int infoIndex = 0;

	while(tmpInfo = nextInfo()){
		ListIterator<molecule*> nextMol(molList);
		molecule** tmpMol;

		while(tmpMol = nextMol()){
			// Is the molecule present in the molecule list?
			if(*tmpMol == tmpInfo->obj.GetMol()){
				// We found the molecule!
				break;
			}
		}

		if(tmpMol == NULL){
			// We didn't find the molecule. Delete this item from 
			// the AtomInfo list
			m_list.Remove(infoIndex);

			// Make a recursive call to Flush to remove additional
			// AtomInfo objects as needed
			Flush(molList);
			return;
		}

		infoIndex++;
	}
}

int measure::NumPicked()
{
	return m_list.Length();
}

BOOL measure::Push(SelectObj &m_obj)
{
	int num_picked = m_list.Length();

	if(num_picked > 0){
		
		// If the user has selected the most recently selected atom,
		// de-select it.
		if(m_obj == m_list[num_picked - 1].obj){
			m_list.Pop();
			return TRUE; // Last atom removed, update the scene
		}
		
		// If the user has selected the first selected atom,
		// de-select it.
		if(m_obj == m_list[0].obj){
			m_list.Remove(0);

			return TRUE; // First atom removed, update the scene
		}
		
		if((num_picked > 2) && (m_obj == m_list[1].obj))
			return FALSE;
		
		if((num_picked > 3) && (m_obj == m_list[2].obj))
			return FALSE;
	}
	
	if(num_picked == 4)
		return FALSE; // No room for more measurements
		
	AtomInfo tmpInfo;
	atom &atomRef = m_obj.GetMol()->atoms[m_obj.GetIndex()];

	tmpInfo.obj = m_obj;
	tmpInfo.serial = atomRef.id;
	tmpInfo.model = atomRef.model;
	
	m_list.Push(tmpInfo);
	
	Update();
	
	return TRUE /* An atom was added */;
}

void measure::Update()
{
	int num_picked = m_list.Length();
	int i;

	if(num_picked == 0)
		return;

	ListIterator<AtomInfo> next(m_list);
	AtomInfo *tmpInfo;
	float tmpCoord[3];

	i = 0;

	while(tmpInfo = next()){
		molecule *tmpMol = tmpInfo->obj.GetMol();
		int tmpIndex = tmpInfo->obj.GetIndex();

		if(tmpMol->current_step == 0){
			atom &atomRef = tmpMol->atoms[tmpIndex];

			tmpCoord[X] = atomRef.coord[X];
			tmpCoord[Y] = atomRef.coord[Y];
			tmpCoord[Z] = atomRef.coord[Z];
		}
		else{
			int trajIndex = tmpMol->current_step - 1;
			tmpIndex *= 3;
			
			tmpCoord[X] = tmpMol->traj[trajIndex][tmpIndex + X];
			tmpCoord[Y] = tmpMol->traj[trajIndex][tmpIndex + Y];
			tmpCoord[Z] = tmpMol->traj[trajIndex][tmpIndex + Z];
		}
		
		TRANSLATE(tmpCoord, tmpMol->R);
		
		// Build the rotation matrix
		float m[4][4];

		tmpMol->BuildRotationMatrix(m);

		x[i] = m[X][X]*tmpCoord[X] + m[Y][X]*tmpCoord[Y] + m[Z][X]*tmpCoord[Z];
		y[i] = m[X][Y]*tmpCoord[X] + m[Y][Y]*tmpCoord[Y] + m[Z][Y]*tmpCoord[Z];
		z[i] = m[X][Z]*tmpCoord[X] + m[Y][Z]*tmpCoord[Y] + m[Z][Z]*tmpCoord[Z];

		i++;
	}
		
	// Update bond lengths
	for(i = 2;i <= num_picked;i++){

		bond_length[i - 2] = GetBond(x[i - 2], y[i - 2], z[i - 2],
									 x[i - 1], y[i - 1], z[i - 1]);
	}
	
	// Update bond angles
	for(i = 3;i <= num_picked;i++)
		bond_angle[i - 3] = GetBondAngle(x[i - 3], y[i - 3], z[i - 3],
										 x[i - 2], y[i - 2], z[i - 2],
									 	 x[i - 1], y[i - 1], z[i - 1]);
	
	// Update the torsion angle
	if(num_picked == 4)
		torsion_angle = GetTorsionAngle(x[0], y[0], z[0],
										x[1], y[1], z[1],
										x[2], y[2], z[2],
									 	x[3], y[3], z[3]);
}

int measure::Index(int i)
{
	#ifdef _DEBUG
	if((i < 0) || (i > 3))
		throw QMOL_ERROR("Index index way out of bounds!");
	#endif // _DEBUG

	if(m_list.Length() > i)
		return m_list[i].obj.GetIndex();
	
	throw QMOL_ERROR("Index out of bounds");
	
	return 0;
}

molecule *measure::GetMol(int i)
{
	#ifdef _DEBUG
	if((i < 0) || (i > 3))
		throw QMOL_ERROR("Index way out of bounds!");
	#endif // _DEBUG

	if(m_list.Length() > i)
		return m_list[i].obj.GetMol();
	
	throw QMOL_ERROR("Index out of bounds");
	
	return NULL;
}

unsigned char *measure::Color(int i)
{
	#ifdef _DEBUG
	if((i < 0) || (i > 3))
		throw QMOL_ERROR("Index way out of bounds!");
	#endif // _DEBUG

	if(m_list.Length() > i)
		return m_list[i].obj.GetMol()->atoms[m_list[i].obj.GetIndex()].rgb_color;
	
	throw QMOL_ERROR("Index out of bounds");
	
	return NULL;
}

float measure::x_coord(int i)
{
	#ifdef _DEBUG
	if((i < 0) || (i > 3))
		throw QMOL_ERROR("x index way out of bounds!");
	#endif // _DEBUG

	if(m_list.Length() > i)
		return x[i];
	
	throw QMOL_ERROR("x index out of bounds");
	
	return 0;
}

float measure::y_coord(int i)
{
	#ifdef _DEBUG
	if((i < 0) || (i > 3))
		throw QMOL_ERROR("y index way out of bounds!");
	#endif // _DEBUG

	if(m_list.Length() > i)
		return y[i];
	
	throw QMOL_ERROR("y index out of bounds");
	
	return 0;	
}

float measure::z_coord(int i)
{
	#ifdef _DEBUG
	if((i < 0) || (i > 3))
		throw QMOL_ERROR("z index way out of bounds!");
	#endif // _DEBUG

	if(m_list.Length() > i)
		return z[i];
	
	throw QMOL_ERROR("z index out of bounds");
	
	return 0;	
}
	
int measure::Id(int i)
{
	#ifdef _DEBUG
	if((i < 0) || (i > 3))
		throw QMOL_ERROR("Id index way out of bounds!");
	#endif // _DEBUG

	if(m_list.Length() > i)
		return m_list[i].serial;
	
	throw QMOL_ERROR("Id index out of bounds");
	
	return 0;	
}

int measure::Model(int i)
{
	#ifdef _DEBUG
	if((i < 0) || (i > 3))
		throw QMOL_ERROR("Model index way out of bounds!");
	#endif // _DEBUG

	if(m_list.Length() > i)
		return m_list[i].model;
	
	throw QMOL_ERROR("Model index out of bounds");
	
	return 0;	
}
	
float measure::Bond(int i)
{
	#ifdef _DEBUG
	if((i < 0) || (i > 2))
		throw QMOL_ERROR("Bond index way out of bounds!");
	#endif // _DEBUG
	
	if(m_list.Length() > (i + 1))
		return bond_length[i];
	
	throw QMOL_ERROR("Bond index out of bounds");
	
	return 0;	
}

float measure::Angle(int i)
{
	#ifdef _DEBUG
	if((i < 0) || (i > 1))
		throw QMOL_ERROR("Angle index way out of bounds!");
	#endif // _DEBUG

	if(m_list.Length() > (i + 2))
		return bond_angle[i];
	
	throw QMOL_ERROR("Angle index out of bounds");
	
	return 0;	
}

float measure::Torsion()
{
	if(m_list.Length() == 4)
		return torsion_angle;
	
	throw QMOL_ERROR("Torsion index out of bounds");
	
	return 0;
}

// Return the bond length between particles i and j.
float GetBond(float x_i, float y_i, float z_i,
	       float x_j, float y_j, float z_j)
{
    float rx = x_j - x_i;
    float ry = y_j - y_i;
    float rz = z_j - z_i;
    
    return (float)(sqrt(rx*rx + ry*ry + rz*rz));
}
	       
// Return the bond angle between particles i, j, k.
float GetBondAngle(float x_i, float y_i, float z_i,
		    float x_j, float y_j, float z_j,
		    float x_k, float y_k, float z_k)
{
	register float ax = x_j - x_i;
	register float ay = y_j - y_i;
	register float az = z_j - z_i;

	register float bx = x_k - x_j;
	register float by = y_k - y_j;
	register float bz = z_k - z_j;
    
	// calculate dot products
	register float ab = dot(ax,ay,az,bx,by,bz);
	register float aa = dot(ax,ay,az,ax,ay,az);
	register float bb = dot(bx,by,bz,bx,by,bz);
    
	//if((aa < PRECISION) || (bb < PRECISION))
	//	throw QMOL_ERROR("Zero length bonds in GetBondAngle");

	/*
 	 * |a||b|cos(theta) = a.b
	 * Calculation could be optimized by replacing the 
	 * sqrt(aa*bb) with r_{ij}*r_{jk}
	 */
	return (float)(PI - acos(ab/sqrt(aa*bb)));
}


/*
 * Return the torsion angle between particles i, j, k and l. 
 * By definition, the torsion angle is positive if the last particle (l)
 * is above the plane defined by the first three particles (i,j,k) - and
 * negative if below. The angle returned is [-Pi, Pi].
 */
float GetTorsionAngle(float x_i, float y_i, float z_i,
		       float x_j, float y_j, float z_j,
		       float x_k, float y_k, float z_k,
		       float x_l, float y_l, float z_l)
{
    register float ax = x_j - x_i;
    register float ay = y_j - y_i;
    register float az = z_j - z_i;

    register float bx = x_k - x_j;
    register float by = y_k - y_j;
    register float bz = z_k - z_j;

    register float cx = x_l - x_k;
    register float cy = y_l - y_k;
    register float cz = z_l - z_k;

    // calculate dot products
    register float ab = dot(ax,ay,az,bx,by,bz);
    register float bc = dot(bx,by,bz,cx,cy,cz);
    register float ac = dot(ax,ay,az,cx,cy,cz);
    register float aa = dot(ax,ay,az,ax,ay,az);
    register float bb = dot(bx,by,bz,bx,by,bz);
    register float cc = dot(cx,cy,cz,cx,cy,cz);

    // Test to see if the torsion angle is defined.
    if((fabs(ab*ab - aa*bb) < PRECISION) ||
       (fabs(bc*bc - bb*cc) < PRECISION)){
		 
		// Changed from a fatal error to a warning 3/2/99 
		// in order to define a molecular connectivity first
		// and internal coordinates later.
		// QMWarning("GetTorsionAngle: Torsion angle undefined and set to zero.");
		return 0.0f;
   }
    // calculate cos(phi)
    register float uu = (aa * bb) - (ab*ab);
    register float vv = (bb * cc) - (bc * bc);
    register float uv = (ab * bc) - (ac * bb);
    register float den= (float)(1.0/sqrt(uu*vv));
    register float co = uv * den;

    /*
     * To fix the problem of round off error making
     * co just slightly larger than 1 and breaking the
     * acos function.
     */
    if(co > 1.0f)
	co = 1.0f;
	
    if(co < -1.0f)
	co = -1.0f;
	
    /*
     * If the determinant of the matrix whose rows are the vectors
     * a, b and c is positive, the torsion angle is positive. Other-
     * wise, the torsion angle is negative.
     */
	if((cx*(ay*bz - az*by) - cy*(ax*bz - az*bx) + cz*(ax*by - ay*bx)) >= 0)
		return (float)(acos(co));
    
	return (float)(-acos(co));
}
