//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/
// 
// Controll the rotation of the molecule(s) with a virtual
// trackball. Based on the trackball code by M. Kilgard.

#ifndef __TRACK_BALL
#define	__TRACK_BALL

// Include the OpenGL headers
#include <GL\gl.h>

// The default trackball size (i.e. mouse senesitivity).
#define DEFAULT_TRACKBALLSIZE  (0.8f)

#define MAX_TRACKBALLSIZE  (1.5f)
#define MIN_TRACKBALLSIZE  (0.4f)

#define RENORMCOUNT 97

class VirtualTrackBall{
	private:
		// Quaterions for the trackball state
		float curquat[4];
		float lastquat[4];

		void axis_to_quat(float a[3], float phi);
		float tb_project_to_sphere(float r, float x, float y);
		float trackballsize;
	public:
		VirtualTrackBall();
		void update();
		void build_rotmatrix(GLfloat m[4][4]);

		void trackball(float p1x, float p1y, 
					   float p2x, float p2y);

		// Allow variable rotational sensitivity by adjusting
		// the size of the virtual track ball.
		void IncreaseBallSize()
		{
			// Don't allow a track ball greater than MAX_TRACKBALLSIZE.
			if(trackballsize < MAX_TRACKBALLSIZE){
				trackballsize += 0.1f;
			}
		};

		void DecreaseBallSize()
		{
			// Don't allow a track ball less than MIN_TRACKBALLSIZE.
			if(trackballsize > MIN_TRACKBALLSIZE){
				trackballsize -= 0.1f;
			}
		};

		void DefaultBallSize()
		{
			trackballsize = DEFAULT_TRACKBALLSIZE;
		};
};

#endif // __TRACK_BALL
